Package: rwicc
Title: Regression with Interval-Censored Covariates
Version: 0.2.0
Authors@R: c(
    person(given = "Douglas Ezra",
           family = "Morrison",
           role = c("aut", "cre","cph"),
           email = "demorrison@ucdavis.edu",
           comment = c(ORCID = "0000-0002-7195-830X")),
    person(given = "Ron",
           family = "Brookmeyer",
           role = c("aut")))
Description: Provides functions to simulate and analyze data for a regression model with an interval censored covariate, as described in Morrison et al. (2021) <doi:10.1111/biom.13472>.
License: MIT + file LICENSE
Encoding: UTF-8
VignetteBuilder: knitr
Config/testthat/edition: 3
Imports: biglm, dplyr (>= 1.1.0), lubridate, stats, lobstr, arm,
        ggplot2, scales, plotly, rlang, ggrepel
Suggests: spelling, rmarkdown, knitr, testthat (>= 3.0.0), markdown,
        pander, vdiffr, withr, badger
Language: en-US
URL: https://d-morrison.github.io/rwicc/,
        https://github.com/d-morrison/rwicc
BugReports: https://github.com/d-morrison/rwicc/issues
Depends: R (>= 4.1.0)
Config/roxygen2/version: 8.0.0
NeedsCompilation: no
Packaged: 2026-05-27 06:40:28 UTC; ezramorrison
Author: Douglas Ezra Morrison [aut, cre, cph] (ORCID:
    <https://orcid.org/0000-0002-7195-830X>),
  Ron Brookmeyer [aut]
Maintainer: Douglas Ezra Morrison <demorrison@ucdavis.edu>
Repository: CRAN
Date/Publication: 2026-05-27 07:10:02 UTC
Built: R 4.6.0; ; 2026-05-27 12:00:34 UTC; unix

# Generated by roxygen2: do not edit by hand

export(build_event_date_possibilities_table)
export(build_omega_table)
export(fit_joint_model)
export(fit_midpoint_model)
export(fit_uniform_model)
export(graph_omega)
export(plot_CDF)
export(plot_censoring_data)
export(plot_phi_curves)
export(simulate_interval_censoring)
importFrom(arm,invlogit)
importFrom(biglm,bigglm)
importFrom(dplyr,all_of)
importFrom(dplyr,any_of)
importFrom(dplyr,bind_rows)
importFrom(dplyr,filter)
importFrom(dplyr,group_by)
importFrom(dplyr,if_else)
importFrom(dplyr,lag)
importFrom(dplyr,left_join)
importFrom(dplyr,mutate)
importFrom(dplyr,n)
importFrom(dplyr,pull)
importFrom(dplyr,reframe)
importFrom(dplyr,select)
importFrom(dplyr,semi_join)
importFrom(dplyr,summarize)
importFrom(dplyr,tibble)
importFrom(ggplot2,aes)
importFrom(ggplot2,element_blank)
importFrom(ggplot2,element_line)
importFrom(ggplot2,element_text)
importFrom(ggplot2,geom_abline)
importFrom(ggplot2,geom_function)
importFrom(ggplot2,geom_line)
importFrom(ggplot2,geom_point)
importFrom(ggplot2,geom_segment)
importFrom(ggplot2,geom_step)
importFrom(ggplot2,ggplot)
importFrom(ggplot2,margin)
importFrom(ggplot2,scale_colour_discrete)
importFrom(ggplot2,scale_linetype_discrete)
importFrom(ggplot2,theme)
importFrom(ggplot2,theme_classic)
importFrom(ggplot2,xlab)
importFrom(ggplot2,xlim)
importFrom(ggplot2,ylab)
importFrom(ggplot2,ylim)
importFrom(lobstr,mem_used)
importFrom(lubridate,days)
importFrom(lubridate,ddays)
importFrom(lubridate,ymd)
importFrom(plotly,ggplotly)
importFrom(rlang,.data)
importFrom(scales,label_parse)
importFrom(stats,binomial)
importFrom(stats,coef)
importFrom(stats,glm)
importFrom(stats,predict)
importFrom(stats,quasibinomial)
importFrom(stats,rbinom)
importFrom(stats,runif)

<!-- README.md is generated from README.Rmd. Please edit that file -->

# rwicc

[![R build
status](https://github.com/d-morrison/rwicc/workflows/R-CMD-check.yaml/badge.svg)](https://github.com/d-morrison/rwicc/actions)
[![](https://img.shields.io/badge/doi-10.5281/zenodo.6600579-green.svg)](https://doi.org/10.5281/zenodo.6600579)
[![](http://cranlogs.r-pkg.org/badges/grand-total/rwicc?color=blue)](https://cran.r-project.org/package=rwicc)
[![](http://cranlogs.r-pkg.org/badges/last-month/rwicc?color=blue)](https://cran.r-project.org/package=rwicc)
[![](https://www.r-pkg.org/badges/version/rwicc?color=blue)](https://cran.r-project.org/package=rwicc)
[![](https://img.shields.io/badge/devel%20version-0.1.2-blue.svg)](https://github.com/d-morrison/rwicc)
[![CRAN
checks](https://badges.cranchecks.info/summary/rwicc.svg)](https://cran.r-project.org/web/checks/check_results_rwicc.html)
[![](https://codecov.io/gh/d-morrison/rwicc/branch/main/graph/badge.svg)](https://app.codecov.io/gh/d-morrison/rwicc)
[![](https://img.shields.io/github/last-commit/d-morrison/rwicc.svg)](https://github.com/d-morrison/rwicc/commits/main)

`rwicc` (“Regression With Interval-Censored Covariates”) is an R
software package implementing an analysis for a regression model
involving an interval-censored covariate, as described in “Regression
with Interval-Censored Covariates: Application to Cross-Sectional
Incidence Estimation” by Morrison, Laeyendecker, and Brookmeyer
(Biometrics, 2021): <https://doi.org/10.1111/biom.13472>.

This analysis uses a joint model for the distributions of the outcome of
interest and the interval-censored covariate, which is treated as a
latent variable; the model parameters are estimated by maximum
likelihood using an EM algorithm. The submodel used for the distribution
of the interval-censored covariate is somewhat specific to the
application of interest (estimation of the mean duration of a
biomarker-defined window period for cross-sectional incidence
estimation), so this package may not be immediately applicable to other
problems. We are publishing it with the goal of making the results in
our paper easier to reproduce and with the hope that others might adapt
pieces of this code for their own applications. Please feel free to
[contact us](mailto:dmorrison01@ucla.edu) with any questions about the
code or the paper!

## Installation

You can install the current released version from
[CRAN](https://cran.r-project.org) with:

``` r
install.packages("rwicc")
```

You can install the development version from
[GitHub](https://github.com/) with:

``` r
install.packages("devtools")
devtools::install_github("d-morrison/rwicc")
```

## Examples of use

See: <https://d-morrison.github.io/rwicc/articles/how-to-use-rwicc.html>

## Code of Conduct

Please note that the rwicc project is released with a [Contributor Code
of Conduct](https://d-morrison.github.io/rwicc/CODE_OF_CONDUCT.html). By
contributing to this project, you agree to abide by its terms.

#' plot estimated and true CDFs for seroconversion date distribution
#'
#' @param true_hazard_alpha The data-generating hazard at the start of the study
#' @param true_hazard_beta The change in data-generating hazard per calendar
#' year
#' @param omega_hat tibble of estimated discrete hazards
#'
#' @returns a ggplot
#' @export
#' @examples
#' \dontrun{
#'
#' hazard_alpha <- 1
#' hazard_beta <- 0.5
#' study_data <- simulate_interval_censoring(
#'   "hazard_alpha" = hazard_alpha,
#'   "hazard_beta" = hazard_beta
#' )
#'
#' # fit model:
#' EM_algorithm_outputs <- fit_joint_model(
#'   obs_level_data = study_data$obs_data,
#'   participant_level_data = study_data$pt_data
#' )
#' plot1 <- plot_CDF(
#'   true_hazard_alpha = hazard_alpha,
#'   true_hazard_beta = hazard_beta,
#'   omega_hat = EM_algorithm_outputs$Omega
#' )
#'
#' print(plot1)
#' }
#'
#' @importFrom dplyr mutate lag filter
#' @importFrom ggplot2 ggplot aes geom_step xlab ylab geom_function
#' @importFrom ggplot2 scale_colour_discrete scale_linetype_discrete theme
#' @importFrom ggplot2 element_text element_blank element_line margin
#' @importFrom lubridate ymd ddays
plot_CDF <- function(true_hazard_alpha,
                     true_hazard_beta,
                     omega_hat) {
  `P(S>s|S>=s,E=e)` <- `P(S>s|E=0)` <- `P(S>=s|E=0)` <- S <- NULL
  cum_haz_fn0 <- function(years_since_study_start) {
    true_hazard_alpha * years_since_study_start +
      true_hazard_beta / 2 * years_since_study_start^2
  }

  cum_haz_fn <- function(years_since_study_start,
                         `years from study start to enrollment`) {
    cum_haz_fn0(years_since_study_start) -
      cum_haz_fn0(`years from study start to enrollment`)
  }

  surv_fn <- function(years_since_study_start,
                      `years from study start to enrollment`) {
    exp(-cum_haz_fn(
      years_since_study_start,
      `years from study start to enrollment`
    ))
  }

  true_model_label <- "Data-generating model"
  est_model_label <- "Estimated model"

  lwd1 <- 1
  omega_hat <- omega_hat |>
    dplyr::mutate(
      "P(S>s|E=0)" = cumprod(`P(S>s|S>=s,E=e)`),
      "P(S>=s|E=0)" = dplyr::lag(`P(S>s|E=0)`, default = 1)
    )

  plot1 <- ggplot2::ggplot(
    ggplot2::aes(
      y = 1 - `P(S>=s|E=0)`,
      x = (S - lubridate::ymd("2001-01-01")) / lubridate::ddays(365)
    ),
    data = omega_hat |> dplyr::filter(S < max(S))
  ) +
    ggplot2::geom_step(
      direction = "hv",
      ggplot2::aes(
        col = est_model_label,
        linetype = est_model_label
      ),
      lwd = lwd1
    ) +
    ggplot2::xlab("Calendar time (years since start of study)") +
    ggplot2::ylab("Probability of seroconversion") +
    ggplot2::geom_function(
      fun = function(x) {
        1 - surv_fn(x, 0)
      },
      ggplot2::aes(
        col = true_model_label,
        linetype = true_model_label
      ),
      lwd = lwd1
    ) +
    ggplot2::scale_colour_discrete("") +
    ggplot2::scale_linetype_discrete("") +
    ggplot2::theme(
      # axis.title.x = ggplot2::element_text(size = 20),
      # axis.text.x = ggplot2::element_text(size = 14),
      # axis.title.y = ggplot2::element_text(size = 20),
      # axis.text.y = ggplot2::element_text(size = 14),
      panel.grid.major = ggplot2::element_blank(),
      panel.grid.minor = ggplot2::element_blank(),
      panel.background = ggplot2::element_blank(),
      axis.line = ggplot2::element_line(colour = "black"),
      legend.position = "top",
      legend.text =
        ggplot2::element_text(
          # size = 14,
          margin = ggplot2::margin(r = 10, unit = "pt")
        )
    )
}

#' Fit a logistic regression model with an interval-censored covariate
#'
#' This function fits a logistic regression model for a binary outcome Y with an
#' interval-censored covariate T, using an EM algorithm, as described in
#' Morrison et al (2021); \doi{10.1111/biom.13472}.

#' @references
#' Morrison, Laeyendecker, and Brookmeyer (2021).
#' "Regression with interval-censored covariates: Application to
#' cross-sectional incidence estimation".
#' Biometrics. \doi{10.1111/biom.13472}.
#'
#' @param participant_level_data a data.frame or tibble with the following
#' variables:
#' * ID: participant ID
#' * E: study enrollment date
#' * L: date of last negative test for seroconversion
#' * R: date of first positive test for seroconversion
#' * Cohort` (optional): this variable can be used to stratify the modeling of
#' the seroconversion distribution.
#' @param obs_level_data a data.frame or tibble with the following variables:
#' * ID: participant ID
#' * O: biomarker sample collection dates
#' * Y: MAA classifications (binary outcomes)
#' @param model_formula the functional form for the regression model for
#' p(y|t) (as a formula() object)

#' @param mu_function a function taking a vector of regression coefficient
#' estimates as input and outputting an estimate of mu (mean duration of
#' MAA-positive infection).

#' @param bin_width the number of days between possible seroconversion dates
#' (should be an integer)

#' @param denom_offset an offset value added to the denominator of the hazard
#' estimates to improve numerical stability

#' @param initial_S_estimate_location determines how seroconversion date is
#' guessed to initialize the algorithm; can be any decimal between 0 and 1;
#' 0.5 = midpoint imputation, 0.25 = 1st quartile, 0 = last negative, etc.

#' @param EM_toler_loglik the convergence cutoff for the log-likelihood
#' criterion ("Delta_L" in the paper)

#' @param EM_toler_est the convergence cutoff for the parameter estimate
#' criterion ("Delta_theta" in the paper)

#' @param coef_change_metric a string indicating the type of parameter estimate
#' criterion to use:
#' \itemize{
#' \item "max abs rel diff coefs" is the "Delta_theta" criterion
#' described in the paper.
#' \item "max abs diff coefs" is the maximum absolute change in
#' the coefficients (not divided by the old values); this criterion can be
#' useful when some parameters are close to 0.
#' \item "diff mu" is the absolute change in mu,
#' which may be helpful in the incidence estimate calibration setting but not
#' elsewhere.
#' }

#' @param EM_max_iterations the number of EM iterations to perform before
#' giving up if still not converged.

#' @param glm_tolerance the convergence cutoff for the glm fit in the M step

#' @param glm_maxit the iterations cutoff for the glm fit in the M step

#' @param verbose whether to print algorithm progress details to the console

#' @returns a list with the following elements:
#' \itemize{
#' \item `Theta`: the estimated regression coefficients for the model of p(Y|T)
#' \item `Mu`: the estimated mean window period (a transformation of `Theta`)
#' \item `Omega`: a table with the estimated parameters for the model of p(S|E).
#' \item `converged`: indicator of whether the algorithm reached its cutoff
#' criteria before reaching the specified maximum iterations. 1 = reached
#' cutoffs, 0 = not.
#' \item `iterations`: the number of EM iterations completed before the
#' algorithm stopped.
#' \item `convergence_metrics`: the four convergence metrics
#' \item `convergence_stats`: a table of the log-likelihood at each iteration
#' }
#'
#' @export
# ==============================================================================

#' @importFrom biglm bigglm
#' @importFrom dplyr summarize n select left_join filter semi_join mutate
#' @importFrom dplyr any_of if_else lag all_of
#' @importFrom lubridate ddays
#' @importFrom stats binomial coef predict glm quasibinomial
#' @importFrom lobstr mem_used

#' @examples
#' \dontrun{
#'
#' # simulate data:
#' study_data <- simulate_interval_censoring()
#'
#' # fit model:
#' EM_algorithm_outputs <- fit_joint_model(
#'   obs_level_data = study_data$obs_data,
#'   participant_level_data = study_data$pt_data
#' )
#' }
fit_joint_model <- function(
  participant_level_data,
  obs_level_data,
  model_formula = stats::formula(Y ~ T),
  mu_function = compute_mu,
  bin_width = 1,
  denom_offset = 0.1,
  EM_toler_loglik = 0.1,
  EM_toler_est = 0.0001,
  EM_max_iterations = Inf,
  glm_tolerance = 1e-7,
  glm_maxit = 20,
  initial_S_estimate_location = 0.25,
  coef_change_metric = "max abs rel diff coefs",
  verbose = FALSE
) {
  # bind the non-standard column name referenced as a bare symbol in the
  # bigglm/glm `weight =` formula below, to avoid an R CMD check
  # "no visible binding for global variable" note:
  `P(S=s|e,l,r,o,y)` <- NULL

  # setup
  {
    # accumulate the per-iteration log-likelihood in an atomic vector;
    # index-extension of an atomic vector is a documented guarantee (unlike
    # tibble row-extension), so this also works for the default
    # EM_max_iterations = Inf
    logL_by_iteration <- numeric(0)

    # starting message
    {
      if (verbose) {
        message(
          "Starting `fit_joint_model();`, mem used = ",
          round(lobstr::mem_used() / 10^6), " MB"
        )
      }
    }

    # check for basic errors in the data
    check_pt_data(participant_level_data)

    # if `Stratum` is not provided in the input data, we create a placeholder:
    if (!is.element("Stratum", colnames(participant_level_data))) {
      participant_level_data$Stratum <- "Cohort 1"
    }

    # identify the set of unique (E,L) combinations and count occurrences
    {
      E_L_combinations <-
        participant_level_data |>
        dplyr::summarize(
          .by = c("Stratum", "E", "L"),
          n_IDs = dplyr::n()
        )
    }

    # identify the set of possible seroconversion dates
    {
      # omega_hat will be a table of possible seroconversion dates, and their
      # estimated hazards, etc:
      omega_hat <-
        participant_level_data |>
        build_omega_table(bin_width = bin_width)

      subj_level_possible_data <-
        participant_level_data |>
        build_event_date_possibilities_table(omega_hat = omega_hat)

      # here we remove from omega_hat any stratum:seroconversion date
      # combinations
      # that aren't in anyone's censoring interval; the best estimate for hazard
      # in those cases is 0, which will drop out of all subsequent calculations:
      omega_hat <- omega_hat |>
        dplyr::semi_join(
          subj_level_possible_data,
          by = c("Stratum", "S")
        )

      # here we enumerate all possible (T,Y) combinations (this tibble gets
      # large):
      obs_data_possibilities <-
        obs_level_data |>
        dplyr::select("ID", "Y", "O") |>
        dplyr::left_join(
          subj_level_possible_data |> dplyr::select("ID", "S"),
          by = "ID"
        ) |>
        dplyr::mutate("T" = (.data$O - .data$S) / lubridate::ddays(365)) |>
        dplyr::select(-"O")
    }

    # count the number of participants definitely at risk of seroconversion
    # (i.e. between E and L) for each time point S:
    {
      omega_hat <- omega_hat |>
        dplyr::left_join(
          E_L_combinations,
          by = "Stratum"
        ) |>
        dplyr::summarize(
          .by = c("Stratum", "S"),
          "n_definitely_at_risk" =
            denom_offset +
              sum(.data$n_IDs[.data$E <= .data$S & .data$S < .data$L])
        )
    }
  }

  # Initial estimates of omega and theta
  {
    # initial guess for S
    {
      participant_level_data <- participant_level_data |>
        dplyr::mutate(
          "S_hat - E" = initial_S_estimate_location * (.data$R - .data$L) +
            (.data$L - .data$E)
        )
      # This duration is computed directly, since computing S_hat first
      # would result in rounding. There's no need for this level of
      # precision in calculating initial estimates, but I am leaving as-is
      # to allow the main-text results to be exactly reproduceable.
    }

    # Omega
    {
      est_hazard_by_stratum <-
        participant_level_data |>
        dplyr::summarize(
          .by = "Stratum",
          "P(S=s|S>=s,E=e)" =
            1 - exp(-lubridate::ddays(bin_width) / mean(.data$`S_hat - E`))
        ) |>
        # this formula actually computes P(S in [s,s+bin_width]|S>=s), from
        # the CDF of an exponential dist. with mean parameter estimated using
        # S_hat - E as approximate event times

        dplyr::mutate("P(S>s|S>=s,E=e)" = 1 - .data$`P(S=s|S>=s,E=e)`)

      omega_hat <- omega_hat |>
        dplyr::left_join(
          est_hazard_by_stratum,
          by = "Stratum"
        )
    }

    # Theta
    {
      obs_level_data <- obs_level_data |>
        dplyr::select(-any_of("E")) |>
        dplyr::left_join(
          participant_level_data |>
            dplyr::select("ID", "S_hat - E", "E"),
          by = "ID"
        ) |>
        dplyr::mutate(
          T0 = ((.data$O - .data$E) - (.data$`S_hat - E`)),
          "T" = .data$T0 / lubridate::ddays(365)
        ) |>
        dplyr::select("ID", "T", "Y")

      MAA_model <- biglm::bigglm(
        formula = model_formula,
        family = stats::binomial(link = "logit"),
        data = obs_level_data,
        maxit = glm_maxit,
        epsilon = glm_tolerance,
        quiet = TRUE
      )
      # quiet = TRUE is used because fitted probabilities numerically 0 or 1 can
      # easily occur, since p(Y=1|T=t) drops quickly to essentially 0; not a
      # cause for concern

      theta.hat <- stats::coef(MAA_model)
      mu.hat <- mu_function(theta.hat)

      if (verbose) {
        message(
          "initial estimate for mu = ",
          round(mu.hat, digits = -log10(EM_toler_est) + 1),
          "; initial estimate for theta:\n"
        )

        print(theta.hat)
      }
    }
  }

  # observed-data likelihood function, with terms that cancel out of
  # diff(logLik) formula removed (see Supporting Information A.3)
  {
    observed_data_log_likelihood <- function(subj_level_possible_data) {
      log_L <- subj_level_possible_data |>
        dplyr::summarize(
          .by = "ID",
          "logL_i" = log(sum(.data$`P(Y=y|T=t)` * .data$`P(S=s|E=e)`))
        ) |>
        dplyr::summarize(
          "log_L" = sum(.data$logL_i)
        )

      return(log_L$log_L)
    }
  }

  # algorithm loop:
  current_iteration <- 0
  while (current_iteration < EM_max_iterations) {
    # progress notifications
    {
      current_iteration <- current_iteration + 1

      if (verbose) {
        message(
          Sys.time(), ": starting EM iteration (E step) ", current_iteration,
          "; mem used = ", round(lobstr::mem_used() / 10^6), " MB"
        )
      }
    }

    # E step: calculate distribution of S, given omega, theta, and (E,L,R,O,Y)
    {
      # calculate P(S>=l|E=e) for each participant:
      {
        E_L_combinations <- E_L_combinations |>
          dplyr::left_join(
            omega_hat |> dplyr::select("Stratum", "S", "P(S>s|S>=s,E=e)"),
            by = "Stratum"
          ) |>
          dplyr::summarize(
            .by = c("Stratum", "E", "L"),
            "P(S>=l|E=e)" = prod(
              .data$`P(S>s|S>=s,E=e)`[.data$E <= .data$S & .data$S < .data$L]
            )
          )
        # note: can't add `filter(E <= S, S < L)` before summarize() or we would
        # lose any (E,L) combinations where E == L.

        participant_level_data <- participant_level_data |>
          dplyr::select(-dplyr::any_of(c("P(S>=l|E=e)"))) |>
          dplyr::left_join(
            E_L_combinations,
            by = c("Stratum", "E", "L")
          )
      }

      # update subj_level_possible_data and obs_data_possibilities:
      {
        subj_level_possible_data <-
          update_possible_subj_data(
            obs_data_possibilities,
            MAA_model,
            participant_level_data,
            omega_hat
          )

        obs_data_possibilities <- obs_data_possibilities |>
          dplyr::select(-dplyr::any_of("P(S=s|e,l,r,o,y)")) |>
          dplyr::left_join(
            subj_level_possible_data |>
              dplyr::select(dplyr::all_of(c("ID", "S", "P(S=s|e,l,r,o,y)"))),
            by = c("ID", "S")
          )
      }

      if (verbose) {
        message("Ending E step.")
      }
    }

    # check for convergence
    {
      # we check for convergence between the E and M step because it is
      # convenient to compute the likelihood as part of the E step

      if (current_iteration > 1) {
        log_L_old <- log_L
      }

      log_L <- observed_data_log_likelihood(subj_level_possible_data)

      logL_by_iteration[current_iteration] <- log_L

      if (verbose) message("observed-data log-likelihood = ", round(log_L, 5))

      if (current_iteration > 1) {
        diff_log_L <- log_L - log_L_old

        if (is.na(diff_log_L)) stop("`diff_log_L` = NA")

        if (diff_log_L < 0) {
          warning(paste("log-likelihood is decreasing; change = ", diff_log_L))
          # note: if denom_offset != 0, we may lose the guarantee of increasing
          # log likelihood.
        }

        # For algorithm diagnostic purposes, we compute three change metrics
        # based on the coefficients, but only one is used to determine
        # convergence, specified in the function inputs; see M step below:
        diff_coef <- coef_diffs[coef_change_metric]

        converged <- (diff_log_L < EM_toler_loglik) &
          (is.na(coef_change_metric) | (diff_coef < EM_toler_est))

        if (converged) {
          if (verbose) {
            message(
              "EM algorithm converged, based on log-likelihood",
              dplyr::if_else(!is.na(coef_change_metric), " and coefs", ""),
              ", to ", diff_log_L, " change in log-likelihood, in ",
              current_iteration, " iterations."
            )
          }

          break
        } else {
          if (verbose) message("Change in log-likelihood = ", diff_log_L)
        }
      }
    }

    # M step: Updated estimation of theta, omega
    {
      if (verbose) {
        message(
          "Starting M step, mem used = ",
          round(lobstr::mem_used() / 10^6), " MB"
        )
      }

      # update omega (hazard model):
      {
        omega_hat_old <- omega_hat

        # sum the estimated at-risk and event probabilities by date:
        n_events_by_date <-
          subj_level_possible_data |>
          dplyr::summarize(
            .by = c("Stratum", "S"),
            "n_events" = sum(.data$`P(S=s|e,l,r,o,y)`),
            "risk_probabilities" = sum(.data$`P(S>=s|e,l,r,o,y)`)
          )

        if (any(with(n_events_by_date, n_events > risk_probabilities))) {
          stop("more events than participants at risk")
        }

        omega_hat <- omega_hat |>
          dplyr::left_join(n_events_by_date, by = c("S", "Stratum")) |>
          dplyr::mutate(
            "n_at_risk" = .data$risk_probabilities + .data$n_definitely_at_risk,
            "P(S=s|S>=s,E=e)" = dplyr::if_else(
              .data$n_at_risk == 0,
              1,
              .data$n_events / .data$n_at_risk
            )
            # if no one is left at risk, estimate the hazard as 1; won't matter
          ) |>
          dplyr::select(-c("risk_probabilities", "n_events", "n_at_risk"))

        # handle numeric stability issues (should not occur if
        # denom_offset > 0):
        {
          # this prevents occasional cases where the denominator approaches 0
          # for the person with the latest seroconversion window

          to_fix <- (omega_hat$"P(S=s|S>=s,E=e)" > 1) |
            is.na(omega_hat$"P(S=s|S>=s,E=e)")
          if (any(to_fix)) {
            message("Fixing ", sum(to_fix), " value(s) of omega_hat with NaNs.")
            omega_hat$"P(S=s|S>=s,E=e)"[to_fix] <- 1
          }
        }

        # compute P(S>s|S>=s,E=e) from P(S=s|S>=s,E=e):
        omega_hat <- omega_hat |>
          dplyr::mutate("P(S>s|S>=s,E=e)" = 1 - .data$`P(S=s|S>=s,E=e)`)
      }

      # update theta (model of MAA classifications, Y~T):
      {
        MAA_model <- try(biglm::bigglm(
          family = stats::binomial(link = "logit"),
          maxit = glm_maxit,
          quiet = TRUE,
          epsilon = glm_tolerance,
          start = theta.hat,
          formula = model_formula,
          data = obs_data_possibilities,
          weight = ~`P(S=s|e,l,r,o,y)`
        ))

        if (inherits(MAA_model, "try-error")) {
          MAA_model <- stats::glm(
            family = stats::quasibinomial(link = "logit"),
            maxit = glm_maxit,
            epsilon = glm_tolerance,
            start = theta.hat,
            formula = model_formula,
            data = obs_data_possibilities,
            weight = `P(S=s|e,l,r,o,y)`
          )
        }

        mu.hat_old <- mu.hat
        mu.hat <- mu_function(stats::coef(MAA_model))

        # we calculate three measures of change in the regression model here;
        # only one is used to assess convergence though ("max abs rel diff
        # coefs", by default):
        coef_diffs <-
          c(
            "diff mu" =
              as.numeric(abs(mu.hat - mu.hat_old)),
            "max abs diff coefs" =
              max(abs(stats::coef(MAA_model) - theta.hat)),
            "max abs rel diff coefs" =
              max(abs(stats::coef(MAA_model) - theta.hat) / abs(theta.hat))
          )

        theta.hat <- stats::coef(MAA_model)
      }

      # report new parameter estimates:
      if (verbose) {
        message(
          "Ending M step; mu = ",
          round(mu.hat, digits = -log10(EM_toler_est) + 1),
          "; theta = \n"
        )

        print(theta.hat)

        message("\nChange in mu = ", coef_diffs["diff mu"])
        message("Max change in theta = ", coef_diffs["max abs diff coefs"])
        message(
          "Max relative change in theta = ",
          coef_diffs["max abs rel diff coefs"]
        )
      }
    }

    if (current_iteration == EM_max_iterations) {
      warning("Maximum iterations reached without convergence.")
    }
  }


  names(mu.hat) <- NULL # otherwise it will be named "(Intercept)"

  to_return <- list(
    Theta = theta.hat,
    Mu = mu.hat,
    Omega = omega_hat,
    converged = as.numeric(converged),
    iterations = current_iteration,
    convergence_metrics = c("diff logL" = diff_log_L, coef_diffs),
    convergence_stats = dplyr::tibble(
      Iteration = seq_along(logL_by_iteration),
      logL = logL_by_iteration
    )
  )

  return(to_return)
}

test_that("fit_joint_model returns the expected structure", {
  withr::local_seed(1)

  study_data <- simulate_interval_censoring(
    study_cohort_size = 30,
    years_in_study = 5,
    probability_of_ever_seroconverting = 1
  )

  # cap iterations for speed; suppress the resulting "max iterations" notice
  # and dplyr's many-to-many join message (this test checks structure, not
  # convergence or messaging)
  result <- suppressWarnings(
    fit_joint_model(
      participant_level_data = study_data$pt_data,
      obs_level_data = study_data$obs_data,
      bin_width = 30,
      EM_max_iterations = 3
    )
  )

  expect_named(
    result,
    c(
      "Theta", "Mu", "Omega", "converged", "iterations",
      "convergence_metrics", "convergence_stats"
    )
  )

  # convergence_stats is built from a per-iteration log-likelihood vector:
  # one row per completed iteration, with Iteration and logL columns.
  expect_s3_class(result$convergence_stats, "data.frame")
  expect_named(result$convergence_stats, c("Iteration", "logL"))
  expect_equal(nrow(result$convergence_stats), result$iterations)
  expect_true(result$iterations >= 1)
})

#' Plot simulated seroconversion density curves
#'
#' Plots example seroconversion-time densities for several intercept/slope
#' hazard combinations.
#'
#' @returns a [ggplot2::ggplot]
#' @keywords internal
graph_simulated_densities <- function() {
  ggplot2::ggplot() +
    ggplot2::theme_classic() +
    ggplot2::theme(
      text = ggplot2::element_text(size = 15),
      legend.position = "bottom"
    ) +
    ggplot2::geom_function(
      fun = seroconversion_density_function,
      args = list(intercept = 0, slope = .5), ggplot2::aes(col = "0+0.5t")
    ) +
    ggplot2::geom_function(
      fun = seroconversion_density_function,
      args = list(intercept = 0, slope = 1), ggplot2::aes(col = "0+1t")
    ) +
    ggplot2::geom_function(
      fun = seroconversion_density_function,
      args = list(intercept = 0, slope = 2), ggplot2::aes(col = "0+2t")
    ) +
    ggplot2::geom_function(
      fun = seroconversion_density_function,
      args = list(intercept = 1, slope = 0), ggplot2::aes(col = "1+ 0t")
    ) +
    ggplot2::geom_function(
      fun = seroconversion_density_function,
      args = list(intercept = 1, slope = .5), ggplot2::aes(col = "1+0.5t")
    ) +
    ggplot2::geom_function(
      fun = seroconversion_density_function,
      args = list(intercept = 10, slope = 0), ggplot2::aes(col = "10+ 0t")
    ) +
    ggplot2::geom_function(
      fun = seroconversion_density_function,
      args = list(intercept = 10, slope = .5), ggplot2::aes(col = "10+0.5t")
    ) +
    ggplot2::xlim(0, 2) +
    ggplot2::xlab("Time since study start (years)") +
    ggplot2::ylim(0, 12) +
    ggplot2::ylab("p(S=s|E=0)") +
    # ggplot2::scale_y_log10() +
    ggplot2::labs(col = "")
}

test_that("simulated-model graphs are consistent", {
  vdiffr::expect_doppelganger(
    "sim-densities",
    graph_simulated_densities()
  )
  vdiffr::expect_doppelganger(
    "sim-hazards",
    graph_simulated_hazards()
  )
  vdiffr::expect_doppelganger(
    "sim-survival-curves",
    graph_simulated_survival_curves()
  )
})

#' rwicc: Regression with Interval-Censored Covariates
#'
#' The \code{rwicc} package implements a regression model with an
#' interval-censored covariate using an EM algorithm, as described in
#' Morrison et al (2021); \doi{10.1111/biom.13472}.
#'
#' @references
#'
#' Morrison, Laeyendecker, and Brookmeyer (2021).
#' "Regression with interval-censored covariates: Application to
#' cross-sectional incidence estimation".
#' Biometrics. \doi{10.1111/biom.13472}.
#'
#' @section rwicc functions:
#' The main \code{rwicc} functions are:
#' \itemize{
#' \item \code{\link{simulate_interval_censoring}}
#' \item \code{\link{fit_joint_model}}
#' }
#'
#' @name rwicc
"_PACKAGE"

#' Create a ggplot with standard theme settings
#'
#' Internal helper function that initializes a ggplot object with consistent
#' theme and styling settings used across multiple plotting functions in the
#' package.
#'
#' @param data A data frame to be passed to [ggplot2::ggplot()].
#'
#' @returns A [ggplot2::ggplot()] object with standard theme settings applied,
#'   including a black and white theme, custom axis styling, and legend
#'   positioning at the bottom.
#'
#' @keywords internal
standard_ggplot <- function(data) {
  data |>
    ggplot2::ggplot() +
    ggplot2::theme_bw() +
    ggplot2::scale_color_discrete(name = "") +
    ggplot2::scale_shape_discrete(name = "") +
    ggplot2::theme(
      panel.border = element_blank(),
      # panel.grid.major = element_blank(),
      panel.grid.minor = element_blank(),
      axis.line = element_line(colour = "black"),
      legend.position = "bottom",
      text = element_text(size = 15)
    )
}

#' Fit model using uniform imputation
#'
#' @param participant_level_data a data.frame or tibble with the following
#' variables:
#' \itemize{
#' \item ID: participant ID
#' \item E: study enrollment date
#' \item L: date of last negative test for seroconversion
#' \item R: date of first positive test for seroconversion
#' \item Cohort` (optional): this variable can be used to stratify the
#' modeling of the seroconversion distribution.
#' }
#' @param obs_level_data a data.frame or tibble with the following variables:
#' \itemize{
#' \item ID: participant ID
#' \item O: biomarker sample collection dates
#' \item Y: MAA classifications (binary outcomes)
#' }
#' @param maxit maximum iterations, passed to \code{bigglm}
#' @param tolerance convergence criterion, passed to \code{bigglm}
#' @param n_imputations number of imputed data sets to create
#'
#' @returns a vector of logistic regression coefficient estimates
#'
#' @export
#'
#' @examples
#' sim_data <- simulate_interval_censoring(
#'   "theta" = c(0.986, -3.88),
#'   "study_cohort_size" = 4500,
#'   "preconversion_interval_length" = 365,
#'   "hazard_alpha" = 1,
#'   "hazard_beta" = 0.5
#' )
#'
#' theta_est_midpoint <- fit_uniform_model(
#'   obs_level_data = sim_data$obs_data,
#'   participant_level_data = sim_data$pt_data
#' )
#'
#' @importFrom biglm bigglm
#' @importFrom dplyr mutate n select any_of left_join
#' @importFrom lubridate ddays
#' @importFrom stats runif binomial coef
fit_uniform_model <- function(participant_level_data,
                              obs_level_data,
                              maxit = 1000,
                              tolerance = 1e-8,
                              n_imputations = 10) {
  L <- R <- ID <- S_imputed <- O <- NULL

  # create a matrix to store the results from each imputed data set
  imputed_coef_ests <- matrix(
    nrow = n_imputations,
    ncol = 2,
    dimnames = list(NULL, c("theta0", "theta1"))
  )

  for (i in 1:n_imputations)
  {
    participant_level_data <- participant_level_data |>
      dplyr::mutate(
        S_imputed =
          L +
            stats::runif(
              n = dplyr::n(),
              min = 0,
              max = (R - L) / lubridate::ddays(1)
            )
      )

    obs_level_data <- obs_level_data |>
      dplyr::select(-dplyr::any_of("S_imputed")) |>
      dplyr::left_join(
        participant_level_data |> dplyr::select("ID", "S_imputed"),
        by = "ID"
      ) |>
      dplyr::mutate(
        T_imputed = (.data$O - .data$S_imputed) / lubridate::ddays(365)
      )

    phi_model_est_imputed <-
      biglm::bigglm(
        epsilon = tolerance,
        maxit = maxit,
        quiet = TRUE,
        data = obs_level_data,
        family = stats::binomial(),
        Y ~ T_imputed
      )

    imputed_coef_ests[i, ] <- stats::coef(phi_model_est_imputed)
  }

  imputed_coef_ests_mean <- colMeans(imputed_coef_ests)
}

#' Seroconversion hazard function
#'
#' Linear instantaneous hazard of seroconversion, `intercept + slope * t`,
#' equal to zero before `entry_time`.
#'
#' @param t numeric vector of times since study start (years)
#' @param intercept hazard at study start
#' @param slope change in hazard per year
#' @param entry_time time of study entry; the hazard is zero before this
#' @returns a numeric vector of hazard values
#' @keywords internal
seroconversion_hazard_function <- function(
  t,
  intercept,
  slope,
  entry_time = 0
) {
  (t >= entry_time) * (intercept + slope * t)
}

#' Seroconversion cumulative hazard function
#'
#' Cumulative hazard obtained by integrating [seroconversion_hazard_function()].
#'
#' @inheritParams seroconversion_hazard_function
#' @returns a numeric vector of cumulative hazard values
#' @keywords internal
seroconversion_cumhaz_function <- function(
  t,
  intercept,
  slope,
  entry_time = 0
) {
  cumhaz <-
    (t > entry_time) * {
      intercept * (t - entry_time) + 1 / 2 * slope * (t^2 - entry_time^2)
    }

  return(cumhaz)
}

#' Seroconversion survival function
#'
#' Survival probability, `exp(-cumulative hazard)`.
#'
#' @param ... arguments passed to [seroconversion_cumhaz_function()]
#' @returns a numeric vector of survival probabilities
#' @keywords internal
seroconversion_survival_function <- function(...) {
  exp(-seroconversion_cumhaz_function(...))
}

#' Seroconversion density function
#'
#' Probability density, `survival * hazard`.
#'
#' @param ... arguments passed to [seroconversion_survival_function()] and
#'   [seroconversion_hazard_function()]
#' @returns a numeric vector of density values
#' @keywords internal
seroconversion_density_function <- function(...) {
  seroconversion_survival_function(...) *
    seroconversion_hazard_function(...)
}

#' convert a pair of simple logistic regression coefficients into P(Y|T) curve:

#'
#' @param coefs numeric vector of coefficients
#' @returns function(t) P(Y=1|T=t)
#' @importFrom arm invlogit
build_phi_function_from_coefs <- function(coefs) {
  function(x) arm::invlogit(coefs[1] + coefs[2] * x)
}


#' compute mean window period duration from simple logistic regression
#' coefficients

#' @param theta numeric vector of coefficients
#' @returns numeric scalar: mean window period duration
compute_mu <- function(theta) {
  365 * -log(exp(theta[1]) + 1) / theta[2]
}

#' Plot censoring data
#'
#' @param dataset output from [simulate_interval_censoring()]
#' @param included_IDs [character] [vector] of IDs from `dataset` to include
#' @param label_size [numeric]:
#' passed to [ggrepel::geom_text_repel()]'s `size` argument
#' @param point_size
#' [numeric]: passed to [ggplot2::geom_point]'s `size` argument
#' @param s_vjust passed to [ggrepel::geom_text_repel]'s `vjust` argument
#' @param labelled_IDs [character] [vector]
#' indicating which IDs to label events for
#' @param xmin minimum displayed value for x-axis
#' @param xmax maximum displayed value for x-axis
#'
#' @returns a ggplot
#' @importFrom ggplot2 ggplot geom_segment geom_point xlab ylab
#' @importFrom plotly ggplotly
#' @importFrom dplyr reframe bind_rows pull
#' @export
#'

plot_censoring_data <- function(
  dataset,
  included_IDs = unique(dataset$pt_data$ID),
  label_size = 5,
  point_size = 5,
  s_vjust = 2,
  labelled_IDs = included_IDs,
  xmin = min(dataset$pt_data$E) - 28,
  xmax = max(dataset$obs_data$O)
) {
  dataset <- dataset |> filter_data_by_ID(included_IDs)

  plot1 <-
    dataset$pt_data |>
    dplyr::filter(.data$ID %in% included_IDs) |>
    standard_ggplot() +
    geom_point(
      size = point_size,
      data = dataset$obs_data0 |>
        filter(.data$`HIV status` == "HIV-"),
      aes(
        x = .data$`O`,
        y = .data$ID,
        col = .data$`HIV status`
      ),
      alpha = .5
    ) +

    ggrepel::geom_text_repel(
      data = dataset$pt_data |>
        filter(.data$ID %in% labelled_IDs),
      size = label_size,
      aes(
        x = .data$E,
        y = .data$ID,
        label = "E"
      ),
      vjust = -1,
      parse = TRUE
    ) +

    ggrepel::geom_text_repel(
      data = dataset$pt_data |>
        filter(.data$ID %in% labelled_IDs),
      size = label_size,
      aes(
        x = .data$L,
        y = .data$ID,
        label = "L"
      ),
      vjust = -1,
      parse = TRUE
    ) +

    ggrepel::geom_text_repel(
      data = dataset$pt_data |> filter(.data$ID %in% labelled_IDs),
      size = label_size,
      aes(
        x = .data$R,
        y = .data$ID,
        label = "R"
      ),
      vjust = -1,
      parse = TRUE
    ) +

    geom_point(
      size = point_size,
      aes(
        x = .data$S,
        y = .data$ID,
        col = "Seroconversion date (S)"
      ),
      alpha = .5
    ) +

    ggrepel::geom_text_repel(
      data = dataset$pt_data |> filter(.data$ID %in% labelled_IDs),
      size = label_size,
      aes(
        x = .data$S,
        y = .data$ID,
        label = "S"
      ),
      vjust = s_vjust,
      parse = TRUE
    ) +


    geom_segment(
      aes(
        x = .data$L,
        xend = .data$R,
        y = .data$ID,
        yend = .data$ID,
        col = "Censoring interval"
      )
    ) +

    geom_point(
      size = point_size,
      data = dataset$obs_data,
      aes(
        x = .data$`O`,
        y = .data$ID,
        col = "HIV+",
        shape = .data$`MAA status`
      ),
      alpha = .5
    ) +

    ggrepel::geom_text_repel(
      data = dataset$obs_data |> filter(.data$ID %in% labelled_IDs),
      size = label_size,
      aes(
        x = .data$`O`,
        y = .data$ID,
        label = paste0("O[", .data$`Obs ID`, "]")
      ),
      vjust = 2,
      min.segment.length = 0,
      parse = TRUE
    ) +

    xlab("Event date") +
    ylab("Participant ID #") +
    xlim(
      xmin,
      xmax
    )

  return(plot1)
}

test_that("results are consistent", {
  withr::local_seed(16)

  sim_data <- simulate_interval_censoring(
    study_cohort_size = 2,
    years_in_study = 10,
    probability_of_ever_seroconverting = 1
  )

  sim_data$obs_data <-
    sim_data$obs_data |>
    filter(.data$O <= lubridate::ymd("2003-01-01"))

  sim_plot <- sim_data |>
    plot_censoring_data(
      labelled_IDs = 1:2,
      s_vjust = -1
    )

  sim_plot |> vdiffr::expect_doppelganger(title = "sim-plot")
})

#' Validate participant-level data
#'
#' Internal helper function that performs basic validation checks on
#' participant-level data to ensure data integrity before model fitting.
#'
#' @param participant_level_data A data.frame or tibble containing
#'   participant-level data with at least the following columns:
#'   * `ID`: participant ID
#'   * `L`: date of last negative test for seroconversion
#'   * `R`: date of first positive test for seroconversion
#'
#' @returns NULL (invisibly). The function throws an error if validation fails.
#'
#' @keywords internal
check_pt_data <- function(participant_level_data) {
  if (with(participant_level_data, any(L > R))) stop("L must be <= R!")

  if (with(participant_level_data, any(duplicated(ID)))) stop("duplicate IDs")
}

#' Fit model using midpoint imputation
#'
#' @param participant_level_data a data.frame or tibble with the following
#' variables:
#' \itemize{
#' \item ID: participant ID
#' \item E: study enrollment date
#' \item L: date of last negative test for seroconversion
#' \item R: date of first positive test for seroconversion
#' \item Cohort` (optional): this variable can be used to stratify the
#' modeling of the seroconversion distribution.
#' }
#' @param obs_level_data a data.frame or tibble with the following variables:
#' \itemize{
#' \item ID: participant ID
#' \item O: biomarker sample collection dates
#' \item Y: MAA classifications (binary outcomes)
#' }
#' @param maxit maximum iterations, passed to \code{bigglm}
#' @param tolerance convergence criterion, passed to \code{bigglm}
#'
#' @returns a vector of logistic regression coefficient estimates
#' @export
#' @examples
#' sim_data <- simulate_interval_censoring(
#'   "theta" = c(0.986, -3.88),
#'   "study_cohort_size" = 4500,
#'   "preconversion_interval_length" = 365,
#'   "hazard_alpha" = 1,
#'   "hazard_beta" = 0.5
#' )
#'
#' theta_est_midpoint <- fit_midpoint_model(
#'   obs_level_data = sim_data$obs_data,
#'   participant_level_data = sim_data$pt_data
#' )
#'
#' @importFrom biglm bigglm
#' @importFrom dplyr mutate left_join select
#' @importFrom lubridate ddays
#' @importFrom stats binomial coef
fit_midpoint_model <- function(participant_level_data,
                               obs_level_data,
                               maxit = 1000,
                               tolerance = 1e-8) {
  # prevent notes about undefined variables due to dplyr syntax:
  L <- R <- ID <- S_midpoint <- O <- NULL

  # bigglm's default maxit = 8, which is not large enough to ensure
  # convergence for this data. I don't think any of the scenarios examined in
  # our paper ever get close to 1000 iterations though; this is effectively
  # saying maxit = Inf.

  participant_level_data <- participant_level_data |>
    dplyr::mutate(S_midpoint = L + (R - L) / lubridate::ddays(2))

  obs_level_data <- obs_level_data |>
    dplyr::left_join(
      by = "ID",
      participant_level_data |> dplyr::select("ID", "S_midpoint")
    ) |>
    dplyr::mutate(
      T_midpoint = (.data$O - .data$S_midpoint) / lubridate::ddays(365)
    )

  phi_model_est_midpoint <-
    biglm::bigglm(
      epsilon = tolerance,
      maxit = maxit,
      quiet = TRUE,
      data = obs_level_data,
      family = stats::binomial(),
      Y ~ T_midpoint
    )

  coefs <- coef(phi_model_est_midpoint)

  return(coefs)
}

#' Plot the estimated seroconversion-date distribution for one participant
#'
#' @param subject_level_data_possibilities a [data.frame] of per-subject
#'   seroconversion-date possibilities with their estimated probabilities
#' @param id the participant ID to plot
#' @returns a [ggplot2::ggplot]
#' @keywords internal
graph_S <- function(
  subject_level_data_possibilities,
  id = 1
) {
  subject_level_data_possibilities |>
    dplyr::filter(.data$ID == id) |>
    ggplot2::ggplot() +
    ggplot2::aes(
      x = .data$S,
      y = .data$`P(S=s|e,l,r,o,y)`,
      col = "P(S=s|e,l,r,o,y)"
    ) +
    ggplot2::geom_point() +
    ggplot2::geom_line() +
    ggplot2::geom_point(
      ggplot2::aes(y = .data$`P(S=s|E=e)`, col = "P(S=s|E=e)")
    ) +
    ggplot2::geom_line(
      ggplot2::aes(y = .data$`P(S=s|E=e)`, col = "P(S=s|E=e)")
    ) +
    ggplot2::geom_point(
      ggplot2::aes(y = .data$`P(Y=y|T=t)`, col = "P(Y=y|T=t)")
    ) +
    ggplot2::geom_line(
      ggplot2::aes(y = .data$`P(Y=y|T=t)`, col = "P(Y=y|T=t)")
    ) +
    ggplot2::xlab("Seroconversion date (s)") +
    ggplot2::ylab("Probability density or mass") +
    ggplot2::theme_classic() +
    ggplot2::theme(
      legend.position = "bottom",
      text = ggplot2::element_text(size = 15)
    )
}

#' Update subject-level data with posterior probabilities
#'
#' Internal helper function that updates subject-level data by computing
#' various conditional probabilities used in the EM algorithm for
#' fitting the joint model. This function combines observation-level
#' predictions with participant-level data and omega estimates to compute
#' posterior probabilities needed for parameter estimation.
#'
#' @param obs_data_possibilities A [data.frame] containing all possible
#'   observation-level data combinations, including ID, S (possible
#'   seroconversion dates), and Y (MAA classifications).
#' @param MAA_model A fitted [stats::glm()] model object used to predict
#'   P(Y=1|T=t), the probability of MAA-positive status given time since
#'   seroconversion.
#' @param participant_level_data A [data.frame] containing participant-level
#'   information, including ID, Stratum, and P(S>=l|E=e) (the probability
#'   that seroconversion occurs on or after the last negative test date).
#' @param omega_hat A [data.frame] containing estimated parameters for the
#'   seroconversion date distribution model, including S, Stratum,
#'   P(S=s|S>=s,E=e), and P(S>s|S>=s,E=e).
#'
#' @returns A [data.frame] containing updated subject-level data with the
#'   following columns:
#' \itemize{
#'   \item `ID`: Participant identifier
#'   \item `Stratum`: Stratification variable
#'   \item `S`: Possible seroconversion date
#'   \item `P(Y=y|T=t)`: Probability of observed MAA outcomes given time since
#'     seroconversion
#'   \item `P(S=s|E=e)`: Marginal probability of seroconversion at time s given
#'     enrollment date
#'   \item `P(S=s|e,l,r,o,y)`: Posterior probability of seroconversion at time s
#'     given all observed data (used to estimate omega and theta)
#'   \item `P(S>=s|e,l,r,o,y)`: Posterior probability that seroconversion occurs
#'     on or after time s (used to estimate omega)
#' }
#'
#' @keywords internal
update_possible_subj_data <- function(
  obs_data_possibilities,
  MAA_model,
  participant_level_data,
  omega_hat
) {
  obs_data_possibilities |>
    dplyr::mutate(
      # could speed up this step by implementing the needed computations
      # explicitly:
      "P(Y=1|T=t)" =
        as.numeric(stats::predict(
          MAA_model,
          newdata = obs_data_possibilities,
          type = "response"
        )),
      "P(Y=y|T=t)" =
        dplyr::if_else(
          .data$Y == 1,
          .data$`P(Y=1|T=t)`,
          1 - .data$`P(Y=1|T=t)`
        )
    ) |>
    dplyr::summarize(
      .by = c("ID", "S"),
      "P(Y=y|T=t)" = prod(.data$`P(Y=y|T=t)`)
    ) |>
    dplyr::left_join(
      participant_level_data |>
        dplyr::select("ID", "Stratum", "P(S>=l|E=e)"),
      by = "ID"
    ) |>
    # update `P(S=s|e,l,r,o,y)`:
    dplyr::left_join(
      omega_hat |> dplyr::select(
        c("S", "Stratum", "P(S=s|S>=s,E=e)", "P(S>s|S>=s,E=e)")
      ),
      by = c("S", "Stratum")
    ) |>
    dplyr::mutate(
      .by = "ID",
      "P(S>s|S>=l,E=e)" = cumprod(.data$`P(S>s|S>=s,E=e)`),
      # used for next calculation

      "P(S>=s|S>=l,E=e)" = dplyr::lag(.data$`P(S>s|S>=l,E=e)`, default = 1),
      # used for next calculation

      "P(S=s|S>=l,E=e)" = .data$`P(S=s|S>=s,E=e)` * .data$`P(S>=s|S>=l,E=e)`,
      # used in next two calculations

      "P(S=s|E=e)" = .data$`P(S=s|S>=l,E=e)` * .data$`P(S>=l|E=e)`,
      # used to compute likelihood

      "P(S=s|E=e,L=l,R=r)" = proportions(.data$`P(S=s|S>=l,E=e)`),
      # used in next calculation

      "P(S=s|e,l,r,o,y)" = proportions(.data$`P(Y=y|T=t)` *
        .data$`P(S=s|E=e,L=l,R=r)`),
      # used to estimate omega and theta

      "P(S>=s|e,l,r,o,y)" = rev(cumsum(rev(.data$`P(S=s|e,l,r,o,y)`)))
      # used to estimate omega
    ) |>
    dplyr::select(
      c(
        "ID",
        "Stratum",
        "S",
        "P(Y=y|T=t)", # used to compute likelihood
        "P(S=s|E=e)", # used to compute likelihood
        "P(S=s|e,l,r,o,y)", # used to estimate omega and theta
        "P(S>=s|e,l,r,o,y)"
      )
    ) # used to estimate omega
}

#' Plot true and estimated curves for P(Y=1|T=t)
#'
#' @param theta_true the coefficients of the data-generating model P(Y=1|T=t)
#' @param theta.hat_joint the estimated coefficients from the joint model
#' @param theta.hat_midpoint the estimated coefficients from midpoint imputation
#' @param theta.hat_uniform the estimated coefficients from uniform imputation
#'
#' @returns a ggplot
#' @export
#' @examples
#' \dontrun{
#'
#' theta_true <- c(0.986, -3.88)
#' hazard_alpha <- 1
#' hazard_beta <- 0.5
#' sim_data <- simulate_interval_censoring(
#'   "theta" = theta_true,
#'   "study_cohort_size" = 4500,
#'   "preconversion_interval_length" = 365,
#'   "hazard_alpha" = hazard_alpha,
#'   "hazard_beta" = hazard_beta
#' )
#'
#' # extract the participant-level and observation-level simulated data:
#' sim_participant_data <- sim_data$pt_data
#' sim_obs_data <- sim_data$obs_data
#' rm(sim_data)
#'
#' # joint model:
#' EM_algorithm_outputs <- fit_joint_model(
#'   obs_level_data = sim_obs_data,
#'   participant_level_data = sim_participant_data,
#'   bin_width = 7,
#'   verbose = FALSE
#' )
#'
#' # midpoint imputation:
#' theta_est_midpoint <- fit_midpoint_model(
#'   obs_level_data = sim_obs_data,
#'   participant_level_data = sim_participant_data
#' )
#'
#' # uniform imputation:
#' theta_est_uniform <- fit_uniform_model(
#'   obs_level_data = sim_obs_data,
#'   participant_level_data = sim_participant_data
#' )
#'
#' plot2 <- plot_phi_curves(
#'   theta_true = theta_true,
#'   theta.hat_uniform = theta_est_uniform,
#'   theta.hat_midpoint = theta_est_midpoint,
#'   theta.hat_joint = EM_algorithm_outputs$Theta
#' )
#'
#' print(plot2)
#' }
#'
#' @importFrom ggplot2 ggplot geom_function aes xlim ylab xlab
#' @importFrom ggplot2 scale_colour_discrete scale_linetype_discrete theme
#' @importFrom ggplot2 element_blank element_line element_text
#' @importFrom scales label_parse
plot_phi_curves <- function(theta_true,
                            theta.hat_joint,
                            theta.hat_midpoint,
                            theta.hat_uniform) {
  mu_true <- compute_mu(theta_true)
  mu_est_EM <- compute_mu(theta.hat_joint)
  mu_est_midpoint <- compute_mu(theta.hat_midpoint)
  mu_est_imputed <- compute_mu(theta.hat_uniform)

  phi_true <- build_phi_function_from_coefs(theta_true)
  phi_est_joint_modeling <- build_phi_function_from_coefs(theta.hat_joint)
  phi_est_midpoint <- build_phi_function_from_coefs(theta.hat_midpoint)
  phi_est_imputed <- build_phi_function_from_coefs(theta.hat_uniform)

  true_model_label2 <- paste(
    sep = "",
    "'   data-generating model;'~mu == '",
    format(mu_true, digits = 1, nsmall = 1),
    "'~days"
  )

  midpoint_label <- paste(
    sep = "",
    "` midpoint imputation;`~hat(mu) == '",
    format(mu_est_midpoint, digits = 1, nsmall = 1),
    "'~days"
  )

  uniform_label <- paste(
    sep = "",
    "`uniform imputation;`~hat(mu) == '",
    format(mu_est_imputed, digits = 1, nsmall = 1),
    "'~days"
  )

  joint_modeling_label <- paste(
    sep = "",
    "'  joint modeling;'~hat(mu) =='",
    format(mu_est_EM, digits = 1, nsmall = 1),
    "'~days"
  )

  lwd1 <- 1


  plot2 <- ggplot2::ggplot(data = NULL) +
    ggplot2::geom_function(
      fun = phi_true,
      lwd = lwd1,
      ggplot2::aes(colour = true_model_label2, linetype = true_model_label2)
    ) +
    ggplot2::geom_function(
      fun = phi_est_midpoint,
      lwd = lwd1,
      ggplot2::aes(colour = midpoint_label, linetype = midpoint_label)
    ) +
    ggplot2::geom_function(
      fun = phi_est_imputed,
      lwd = lwd1,
      ggplot2::aes(colour = uniform_label, linetype = uniform_label)
    ) +
    ggplot2::geom_function(
      fun = phi_est_joint_modeling,
      lwd = lwd1,
      ggplot2::aes(
        colour = joint_modeling_label, linetype = joint_modeling_label
      )
    ) +
    ggplot2::xlim(0, 2) +
    ggplot2::ylab("Probability of an MAA-positive blood sample") +
    ggplot2::xlab("Time since seroconversion (years)") +
    ggplot2::scale_colour_discrete("", labels = scales::label_parse()) +
    ggplot2::scale_linetype_discrete("", labels = scales::label_parse()) +
    ggplot2::theme(
      panel.grid.major = ggplot2::element_blank(),
      panel.grid.minor = ggplot2::element_blank(),
      panel.background = ggplot2::element_blank(),
      axis.line = ggplot2::element_line(colour = "black")
    ) +
    ggplot2::theme(
      # axis.title.x = ggplot2::element_text(size = 20),
      # axis.text.x = ggplot2::element_text(size = 14),
      # axis.title.y = ggplot2::element_text(size = 18),
      # axis.text.y = ggplot2::element_text(size = 14),
      legend.text.align = 1,
      legend.box.just = "right",
      # legend.text = ggplot2::element_text(size = 16),
      legend.position = c(.65, .85),
      legend.title = ggplot2::element_blank()
    )
}

#' Graph seroconversion hazard model
#'
#' @param omega a [data.frame] containing parameter values for
#' the seroconversion hazard model
#'
#' @returns a [ggplot2::ggplot]
#' @export
#'
#' @examples
#' example_model <-
#'   system.file("extdata", "example_model.rds", package = "rwicc") |>
#'   readRDS()
#' omega_est_EM <- example_model$Omega
#' omega_est_EM |> graph_omega()
#'
graph_omega <- function(omega) {
  omega |>
    ggplot2::ggplot() +
    ggplot2::aes(
      x = .data$S,
      y = .data$`P(S=s|S>=s,E=e)`
    ) +
    ggplot2::geom_point(
      data = omega |> dplyr::filter(.data$`P(S=s|S>=s,E=e)` > 0),
      alpha = .5
    ) +
    ggplot2::geom_line(alpha = .5) +
    ggplot2::theme_classic() +
    ggplot2::xlab("Seroconversion date (s)") +
    ggplot2::ylab("P(S=s|S>=s)") +
    ggplot2::theme(
      text = ggplot2::element_text(size = 15)
    )
}

test_that("results are consistent", {
  example_model <-
    system.file("extdata", "example_model.rds", package = "rwicc") |>
    readRDS()
  omega_est_EM <- example_model$Omega
  omega_est_EM |>
    graph_omega() |>
    vdiffr::expect_doppelganger(title = "example-model")
})

#' Plot simulated seroconversion hazard curves
#'
#' Plots example linear seroconversion hazards for several intercept/slope
#' combinations.
#'
#' @returns a [ggplot2::ggplot]
#' @keywords internal
graph_simulated_hazards <- function() {
  ggplot2::ggplot() +
    ggplot2::theme_classic() +
    ggplot2::theme(
      text = ggplot2::element_text(size = 15),
      legend.position = "bottom"
    ) +
    ggplot2::geom_abline(
      ggplot2::aes(intercept = 0, slope = .5, col = "0+0.5t")
    ) +
    ggplot2::geom_abline(
      ggplot2::aes(intercept = 0, slope = 1, col = "0+1t")
    ) +
    ggplot2::geom_abline(
      ggplot2::aes(intercept = 0, slope = 2, col = "0+2t")
    ) +
    ggplot2::geom_abline(
      ggplot2::aes(intercept = 1, slope = 0, col = "1+ 0t")
    ) +
    ggplot2::geom_abline(
      ggplot2::aes(intercept = 1, slope = .5, col = "1+0.5t")
    ) +
    ggplot2::geom_abline(
      ggplot2::aes(intercept = 10, slope = 0, col = "10+ 0t")
    ) +
    ggplot2::geom_abline(
      ggplot2::aes(intercept = 10, slope = .5, col = "10+0.5t")
    ) +
    ggplot2::xlim(0, 2) +
    ggplot2::ylab("Hazard, p(S=s|S \u2265 s)") +
    ggplot2::xlab("Time since study start (s, years)") +
    ggplot2::ylim(0, 12) +
    ggplot2::labs(col = "")
}

#' @keywords internal
"_PACKAGE"

## usethis namespace: start
#' @importFrom dplyr reframe
#' @importFrom ggplot2 aes
#' @importFrom ggplot2 element_text
#' @importFrom ggplot2 geom_abline
#' @importFrom ggplot2 geom_line
#' @importFrom ggplot2 geom_point
#' @importFrom ggplot2 ggplot
#' @importFrom ggplot2 theme
#' @importFrom ggplot2 theme_classic
#' @importFrom ggplot2 xlab
#' @importFrom ggplot2 xlim
#' @importFrom ggplot2 ylim
#' @importFrom rlang .data
## usethis namespace: end
NULL

#' Filter dataset by participant IDs
#'
#' Internal helper function that filters all data frames within a dataset
#' structure to include only the specified participant IDs.
#'
#' @param dataset A list containing data frames output from
#'   [simulate_interval_censoring()], including `pt_data`, `obs_data0`,
#'   and `obs_data` components.
#' @param included_IDs A [character] [vector] of participant IDs to retain
#'   in the filtered dataset.
#'
#' @returns A list with the same structure as `dataset`, but with all
#'   data frames filtered to include only rows where the ID is in
#'   `included_IDs`.
#'
#' @keywords internal
filter_data_by_ID <- function(dataset, included_IDs) {
  dataset$pt_data <- dataset$pt_data |>
    dplyr::filter(.data$ID %in% included_IDs)
  dataset$obs_data0 <- dataset$obs_data0 |>
    dplyr::filter(.data$ID %in% included_IDs)
  dataset$obs_data <- dataset$obs_data |>
    dplyr::filter(.data$ID %in% included_IDs)

  return(dataset)
}

#' Inverse survival function for time-to-event variable with linear hazard
#' function
#'
#' This function determines the seroconversion date corresponding to a provided
#' probability of survival. See \doi{10.1111/biom.13472},
#' Supporting Information, Section A.4.
#'
#' @references
#'
#' Morrison, Laeyendecker, and Brookmeyer (2021).
#' "Regression with interval-censored covariates: Application to
#' cross-sectional incidence estimation".
#' Biometrics, \doi{10.1111/biom.13472}.

#' @param u a vector of seroconversion survival probabilities
#' @param e a vector of time differences between study start and enrollment
#' (in years)
#' @param hazard_alpha the instantaneous hazard of seroconversion on the study
#' start date
#' @param hazard_beta the change in hazard per year after study start date

#' @returns numeric vector of time differences between study start and
#' seroconversion (in years)
###############################################################################

seroconversion_inverse_survival_function <- function(
  u,
  e,
  hazard_alpha,
  hazard_beta
) {
  if (hazard_beta != 0) {
    a <- hazard_beta / 2
    b <- hazard_alpha
    c <- log(u) - ((a * (e^2)) + (b * e))

    # this is the quadratic formula:
    root <- (-b + sqrt(b^2 - 4 * a * c)) / (2 * a)

    return(root)
  } else {
    value <- e + (-log(u) / hazard_alpha)
    # equivalent:
    # value = e + qexp(p = u, rate = hazard_alpha, lower.tail = FALSE)
    return(value)
  }
}

#' Build table of dates with possible seroconversions in dataset
#'
#' @param participant_level_data a [data.frame] with columns:
#' * Stratum: indicator for which population stratum the participant belongs to
#' * L: left censoring interval endpoint
#' * R: right censoring interval endpoint
#' @param bin_width number of days per bin
#'
#' @returns a [data.frame]
#' @export
#'
build_omega_table <- function(
  participant_level_data,
  bin_width = 1
) {
  participant_level_data |>
    dplyr::reframe(
      .by = "Stratum",
      S = seq(min(.data$L), max(.data$R), by = bin_width)
    )
}

#' Simulate a dataset with interval-censored seroconversion dates
#'
#' \code{simulate_interval_censoring} generates a simulated data set from a
#' data-generating model based on the typical structure of a cohort study of HIV
#' biomarker progression, as described in Morrison et al (2021);
#' \doi{10.1111/biom.13472}.
#'
#' @references
#'
#' Morrison, Laeyendecker, and Brookmeyer (2021).
#' "Regression with interval-censored covariates: Application to
#' cross-sectional incidence estimation".
#' Biometrics. \doi{10.1111/biom.13472}.
#'
#' @param study_cohort_size the number of participants to simulate (N_0 in the
#'   paper)
#' @param hazard_alpha the hazard (instantaneous risk) of seroconversion at the
#'   start date of the cohort study for those participants at risk of
#'   seroconversion
#' @param hazard_beta the change in hazard per calendar year
#' @param preconversion_interval_length the number of days between tests for
#'   seroconversion
#' @param theta the parameters of a logistic model (with linear functional from)
#'   specifying the probability of MAA-positive biomarkers as a function of time
#'   since seroconversion
#' @param probability_of_ever_seroconverting the probability that each
#'   participant is at risk of HIV seroconversion
#' @param years_in_study the duration of follow-up for each participant
#' @param max_scheduling_offset the maximum divergence of pre-seroconversion
#'   followup visits from the prescribed schedule
#' @param days_from_study_start_to_recruitment_end the length of the recruitment
#'   period
#' @param study_start_date the date when the study starts recruitment ("d_0" in
#'   the main text). The value of this parameter does not affect the simulation
#'   results; it is only necessary as a reference point for generating E, L,
#'   R, O, and S.
#' @param n_at_risk number of participants who are at risk of infection; by
#'   default, this number is determined stochastically from
#'   `study_cohort_size` and `probability_of_ever_seroconverting`.

#' @returns A list containing the following two tibbles:
#'
#' * `pt_data`: a tibble of participant-level information, with the following
#'   columns:
#'   - `ID`: participant ID
#'   - `E`: enrollment date
#'   - `L`: date of last HIV test prior to seroconversion
#'   - `R`: date of first HIV test after seroconversion
#'
#' * `obs_data`: a tibble of longitudinal observations with the following
#'   columns:
#'   - `ID`: participant ID
#'   - `O`: dates of biomarker sample collection
#'   - `Y`: MAA classifications of biomarker samples
#' @export
#'
#' @examples
#' study_data <- simulate_interval_censoring()
#' participant_characteristics <- study_data$pt_data
#' longitudinal_observations <- study_data$obs_data
#' @importFrom dplyr tibble if_else filter group_by summarize mutate n select
#' @importFrom lubridate days ddays
#' @importFrom stats rbinom runif
simulate_interval_censoring <- function(
  study_cohort_size = 4500,
  probability_of_ever_seroconverting = 0.05,
  n_at_risk = stats::rbinom(
    n = 1,
    size = study_cohort_size,
    prob = probability_of_ever_seroconverting
  ),
  hazard_alpha = 1,
  hazard_beta = 0.5,
  preconversion_interval_length = 84,
  theta = c(0.986, -3.88),
  years_in_study = 10,
  max_scheduling_offset = 7,
  days_from_study_start_to_recruitment_end = 365,
  study_start_date = lubridate::ymd("2001-01-01")
) {
  # this bit of code just removes some notes produced by check(); see
  # https://r-pkgs.org/package-within.html?q=no%20visible%20binding#echo-a-working-package
  ID <- E <- L <- R <- O <- Y <- S <- `exit date` <- `elapsed time` <-
    `years from study start to sample date` <-
    `years from study start to seroconversion` <- NULL

  # define p(Y=1|T=t):
  phi <- build_phi_function_from_coefs(theta)

  # define the sequence of post-diagnosis biomarker collection dates:
  post_seroconversion_obs_dates <-
    c(
      seq(0, 12 * 7, by = 28),
      # 0, 4, 8, and 12 weeks post-diagnosis
      seq(12 * 14, 365 * 2, by = 12 * 7),
      # then at 12 week intervals until two years
      seq(365 * 2, years_in_study * 365, by = 365)
    ) # then at 1 year intervals until drop-out or study end.

  # generate the number of study cohort participants who are at risk of
  # infection; this step takes the place of assigning A_i to each cohort
  # participant in the main text (the final data set only includes participants
  # with A_i = 1):


  # generate E (enrollment date), F (exit date), S (seroconversion date):
  sim_participant_data <- dplyr::tibble(
    "ID" = factor(seq_len(n_at_risk)),
    "E" =
      study_start_date +
        lubridate::days(
          sample(
            x = 0:days_from_study_start_to_recruitment_end,
            size = n_at_risk,
            replace = TRUE
          )
        ),
    `exit date` = E + years_in_study * lubridate::days(365),
    "years from study start to seroconversion" =
      seroconversion_inverse_survival_function(
        u = stats::runif(n = n_at_risk),
        e = (E - study_start_date) / lubridate::ddays(365),
        hazard_alpha = hazard_alpha,
        hazard_beta = hazard_beta
      ),
    S = study_start_date +
      .data$`years from study start to seroconversion` * 365
    # note that this variable is rounded down at the day level; to compute T
    # below we will use the non-rounded value from `years from study start to
    # seroconversion`
  )

  sim_obs_data0 <- NULL

  # generate L, R:
  for (i in rownames(sim_participant_data))
  {
    ID <- sim_participant_data[i, "ID", drop = TRUE]
    E <- sim_participant_data[i, "E", drop = TRUE]
    S <- sim_participant_data[i, "S", drop = TRUE]

    preconversion_obs_dates <-
      E +
      seq(0, years_in_study * 365, by = preconversion_interval_length)

    # generate small random offsets from schedule in followup dates; when one
    # checkup is later than planned, the next is scheduled relative to last
    # checkup, rather than reverting to schedule; hence offsets accumulate:
    offsets <- cumsum(sample(
      x = (-max_scheduling_offset):max_scheduling_offset,
      size = length(preconversion_obs_dates) - 1,
      replace = TRUE
    ))

    preconversion_obs_dates[-1] <-
      preconversion_obs_dates[-1] + offsets

    sim_participant_data[i, "L"] <-
      max(preconversion_obs_dates[preconversion_obs_dates <= S])

    sim_participant_data[i, "R"] <-
      dplyr::if_else(
        any(preconversion_obs_dates > S),
        min(preconversion_obs_dates[preconversion_obs_dates > S]),
        as.Date(NA)
      )

    temp <-
      sim_participant_data[i, c("ID", "E", "R")] |>
      reframe(
        `Obs ID` = seq_along(preconversion_obs_dates),
        O = preconversion_obs_dates,
        .by = c("ID", "E", "R")
      ) |>
      dplyr::filter(O <= R) |>
      dplyr::mutate(
        `HIV status` =
          if_else(O >= R, "HIV+", "HIV-") |>
            factor() |>
            stats::relevel(ref = "HIV-")
      ) |>
      dplyr::select(-"R")


    sim_obs_data0 <-
      sim_obs_data0 |>
      bind_rows(temp)
  }

  # remove participants who wouldn't be diagnosed until after their their
  # followup period ends:
  sim_participant_data <- sim_participant_data |>
    dplyr::filter(R <= `exit date`)

  # # generate O, Y:
  sim_obs_data <-
    sim_participant_data |>
    dplyr::reframe(
      .by = c(
        "ID", "E", "R", "S", "exit date",
        "years from study start to seroconversion"
      ),
      `Obs ID` = seq_along(post_seroconversion_obs_dates),
      "O" = R + post_seroconversion_obs_dates
    ) |>
    # everyone gets 10 years of observation, total, no longer how long they took
    # to seroconvert:
    dplyr::filter(O <= `exit date`) |>
    dplyr::mutate(
      # we use relative times in order to calculate phi(t) with t = the elapsed
      # time from the exact moment of seroconversion until the date of testing
      # (we assume that all testing occurs at midnight at the beginning of the
      # scheduled day):
      "years from study start to sample date" =
        (O - study_start_date) / lubridate::ddays(365),
      "elapsed time" =
        `years from study start to sample date` -
          `years from study start to seroconversion`,
      "Y" =
        stats::rbinom(
          size = 1,
          n = dplyr::n(),
          p = phi(`elapsed time`)
          # could switch to rbernoulli here, but doing so would change a few of
          # the generated values
        ) |>
          as.numeric(),
      `MAA status` =
        if_else(
          Y == 1,
          "MAA+",
          "MAA-"
        ) |>
          factor(levels = c("MAA-", "MAA+"))
    )

  # remove variables not needed for analysis:
  {
    sim_participant_data <- sim_participant_data |>
      dplyr::select("ID", "E", "L", "R", "S")
    sim_obs_data <- sim_obs_data |>
      dplyr::select("ID", "E", "O", "Y", "S", "MAA status", "Obs ID")
  }

  return(list(
    obs_data0 = sim_obs_data0,
    obs_data = sim_obs_data,
    pt_data = sim_participant_data
  ))
}

#' Build table of event date possibilities
#'
#' @param participant_level_data a [data.frame] with columns:
#' * ID: participant identifier
#' * Stratum: indicator for which population stratum the participant belongs to
#' * L: left censoring interval endpoint
#' * R: right censoring interval endpoint
#' @param omega_hat a [data.frame] from [build_omega_table()] representing the
#' seroconversion hazard model.
#' @inheritParams  build_omega_table
#'
#' @returns a [data.frame] with columns:
#' * ID: participant identifier
#' * Stratum: indicator for which population stratum the participant belongs to
#' * S: possible seroconversion dates
#' @export
#'
#' @examples
#' library(dplyr)
#' simulate_interval_censoring()$pt_data |>
#'   mutate(Stratum = 1) |>
#'   build_event_date_possibilities_table()
build_event_date_possibilities_table <- function(
  participant_level_data,
  bin_width = 1,
  omega_hat =
    participant_level_data |>
      build_omega_table(bin_width = bin_width)
) {
  participant_level_data |>
    dplyr::select("ID", "Stratum", "L", "R") |>
    dplyr::left_join(omega_hat, by = "Stratum") |>
    dplyr::filter(.data$L <= .data$S, .data$S <= .data$R) |>
    dplyr::select(-c("L", "R"))
}

---
title: "visualize-study-design"
output: rmarkdown::html_vignette
vignette: >
  %\VignetteIndexEntry{visualize-study-design}
  %\VignetteEncoding{UTF-8}
  %\VignetteEngine{knitr::rmarkdown}
editor_options: 
  chunk_output_type: console
---

```{r, include = FALSE}
knitr::opts_chunk$set(
  collapse = TRUE,
  comment = "#>",
  fig.width = 8,
  fig.height = 6
  
)
```

```{r setup}
library(rwicc)

```

This vignette demos the study design.

```{r}

set.seed(16); 
simulate_interval_censoring(
    study_cohort_size = 2, 
    years_in_study = 10, 
    probability_of_ever_seroconverting = 1) |> 
    plot_censoring_data(
      included_IDs = 1:2,
      labelled_IDs = 1,
      s_vjust = -1,
      xmax = lubridate::ymd("2003-01-01")
    ) |> 
    print()

```

---
title: "how-to-use-rwicc"
output: rmarkdown::html_vignette
vignette: >
  %\VignetteIndexEntry{how-to-use-rwicc}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
---


```{r, include = FALSE}

library(knitr)

knitr::opts_chunk$set(
  collapse = TRUE,
  comment = "#>"
)
```

This vignette shows how to generate a simulated data set and analyze it using the model and estimation algorithm
described in "Regression with Interval-Censored Covariates: Application to Cross-Sectional Incidence
Estimation" by Morrison, Laeyendecker, and Brookmeyer (2021) in Biometrics: https://onlinelibrary.wiley.com/doi/10.1111/biom.13472.

First, we simulate some data:

```{r}

set.seed(1)

library(rwicc)
library(dplyr)
library(ggplot2)
theta_true = c(0.986, -3.88)
hazard_alpha = 1
hazard_beta = 0.5
sim_data = simulate_interval_censoring(
  "theta" = theta_true,
  "study_cohort_size" = 4500,
  "preconversion_interval_length" = 365,
  "hazard_alpha" = hazard_alpha,
  "hazard_beta" = hazard_beta
)

# extract the participant-level and observation-level simulated data:
sim_participant_data = sim_data$pt_data
sim_obs_data = sim_data$obs_data

```

Here's a look at the first few rows of participant-level data:

```{r}
library(pander)
pander(head(sim_participant_data))
```

* `E` is the individual's enrollment date
* `L` is the date of the last HIV-negative test
* `R` is the date of the first HIV-positive test

We can count the number of individuals whose censoring windows overlap with each day like so:

```{r}
sim_participant_data |>
  mutate("Stratum" = 1) |> 
  build_event_date_possibilities_table(bin_width = 1) |> 
  dplyr::count(S) |> 
  ggplot(
    aes(x = S, y = n)
  ) +
  geom_line(alpha = .5) +
  theme_classic()
```


Next, let's look at the first few rows of observation-level (longitudinal) data:

```{r}
pander(head(sim_obs_data))
```

* `O` is the observation date
* `Y` is the MAA classification (1 = "recent infection", 0 = "long-term infection")

The two tables are linked by the variable `ID`.

We can visualize the data:

```{r}
sim_data |> 
plot_censoring_data(
        included_IDs = 1:10,
        labelled_IDs = NULL,
        # xmax = lubridate::ymd("2003-01-01"),
        s_vjust = -1
    ) +
  theme(axis.ticks.y = element_blank())
```


Now, we will apply our proposed analysis (this takes a couple of minutes; use argument `verbose = TRUE` to print progress messages):

```{r}
EM_algorithm_outputs = fit_joint_model(
  obs_level_data = sim_obs_data,
  participant_level_data = sim_participant_data,
  bin_width = 7,
  EM_max_iterations = 150,
  verbose = FALSE)

```

The output of `fit_joint_model()` is a list with several components:

```{r}
names(EM_algorithm_outputs)
```

`Theta` is the vector of estimated logistic regression coefficients for $P(Y|T)$ (intercept and slope):
```{r}

pander(EM_algorithm_outputs$Theta)
```

`Mu` is the corresponding $\hat{\mu}$ estimate:

```{r}
mu_est_EM = EM_algorithm_outputs$Mu
print(mu_est_EM)
```

`Omega` is the corresponding $\hat{\omega}(s)$ estimate:

```{r}
omega_est_EM <- EM_algorithm_outputs$Omega

omega_est_EM |> graph_omega()

```

`converged` indicates whether the algorithm reached its convergence criterion (= 1 if converged, = 0 if not).

```{r}
EM_algorithm_outputs$converged
```

`iterations` is the number of EM iterations that the algorithm performed:
```{r}
EM_algorithm_outputs$iterations
```

`convergence_metrics` gives the values of all four metrics that we might use to evaluate convergence:

* `diff logL`: change in log-likelihood between iterations
* `diff mu`: change in $\hat{\mu}$
* `max abs diff coefs`: $\max_{j\in 0:1} \{|\hat{\theta}_j^{(k)} - \hat{\theta}_j^{(k-1)}|\}$
* `max abs rel diff coefs`: $\max_{j\in 0:1} \{|(\hat{\theta}_j^{(k)} - \hat{\theta}_j^{(k-1)})/\hat{\theta}_j^{(k-1)}|\}$

By default, the convergence criterion is: `diff logL` < 0.1 and `max abs rel diff coefs` < 0.0001.

```{r}
pander(EM_algorithm_outputs$convergence_metrics)
```

Next, we perform an alternative analysis using midpoint imputation:

```{r}

theta_est_midpoint = fit_midpoint_model(
  obs_level_data = sim_obs_data,
  participant_level_data = sim_participant_data
)

pander(theta_est_midpoint)
```

Here, we perform an alternative analysis using uniform imputation:
```{r}
# uniform imputation:
theta_est_uniform = fit_uniform_model(
  obs_level_data = sim_obs_data,
  participant_level_data = sim_participant_data
)

pander(theta_est_uniform)
```

Now, let's graph the results. First, let's plot the true and estimated CDFs for the distribution of seroconversion date,
for individuals who enroll on the first calendar day of the cohort study:


```{r, fig.width = 6, fig.asp = .75}

plot1 = plot_CDF(
  true_hazard_alpha = hazard_alpha,
  true_hazard_beta = hazard_beta,
  omega_hat = EM_algorithm_outputs$Omega)

print(plot1)

```

We can see that our joint modeling approach hasn't estimated this distribution very accurately for this particular simulated dataset.
Nevertheless, the next graph will show us that the joint model very accurately estimates the true distribution $P(Y|T)$
and the true value of $\mu$:

```{r, fig.width = 6, fig.asp = .8}

plot2 = plot_phi_curves(
  theta_true = theta_true,
  theta.hat_uniform = theta_est_uniform,
  theta.hat_midpoint = theta_est_midpoint,
  theta.hat_joint = EM_algorithm_outputs$Theta)

print(plot2)
```

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/plot_phi_curves.R
\name{plot_phi_curves}
\alias{plot_phi_curves}
\title{Plot true and estimated curves for P(Y=1|T=t)}
\usage{
plot_phi_curves(
  theta_true,
  theta.hat_joint,
  theta.hat_midpoint,
  theta.hat_uniform
)
}
\arguments{
\item{theta_true}{the coefficients of the data-generating model P(Y=1|T=t)}

\item{theta.hat_joint}{the estimated coefficients from the joint model}

\item{theta.hat_midpoint}{the estimated coefficients from midpoint imputation}

\item{theta.hat_uniform}{the estimated coefficients from uniform imputation}
}
\value{
a ggplot
}
\description{
Plot true and estimated curves for P(Y=1|T=t)
}
\examples{
\dontrun{

theta_true <- c(0.986, -3.88)
hazard_alpha <- 1
hazard_beta <- 0.5
sim_data <- simulate_interval_censoring(
  "theta" = theta_true,
  "study_cohort_size" = 4500,
  "preconversion_interval_length" = 365,
  "hazard_alpha" = hazard_alpha,
  "hazard_beta" = hazard_beta
)

# extract the participant-level and observation-level simulated data:
sim_participant_data <- sim_data$pt_data
sim_obs_data <- sim_data$obs_data
rm(sim_data)

# joint model:
EM_algorithm_outputs <- fit_joint_model(
  obs_level_data = sim_obs_data,
  participant_level_data = sim_participant_data,
  bin_width = 7,
  verbose = FALSE
)

# midpoint imputation:
theta_est_midpoint <- fit_midpoint_model(
  obs_level_data = sim_obs_data,
  participant_level_data = sim_participant_data
)

# uniform imputation:
theta_est_uniform <- fit_uniform_model(
  obs_level_data = sim_obs_data,
  participant_level_data = sim_participant_data
)

plot2 <- plot_phi_curves(
  theta_true = theta_true,
  theta.hat_uniform = theta_est_uniform,
  theta.hat_midpoint = theta_est_midpoint,
  theta.hat_joint = EM_algorithm_outputs$Theta
)

print(plot2)
}

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/graph_omega.R
\name{graph_omega}
\alias{graph_omega}
\title{Graph seroconversion hazard model}
\usage{
graph_omega(omega)
}
\arguments{
\item{omega}{a \link{data.frame} containing parameter values for
the seroconversion hazard model}
}
\value{
a \link[ggplot2:ggplot]{ggplot2::ggplot}
}
\description{
Graph seroconversion hazard model
}
\examples{
example_model <-
  system.file("extdata", "example_model.rds", package = "rwicc") |>
  readRDS()
omega_est_EM <- example_model$Omega
omega_est_EM |> graph_omega()

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/filter_data_by_ID.R
\name{filter_data_by_ID}
\alias{filter_data_by_ID}
\title{Filter dataset by participant IDs}
\usage{
filter_data_by_ID(dataset, included_IDs)
}
\arguments{
\item{dataset}{A list containing data frames output from
\code{\link[=simulate_interval_censoring]{simulate_interval_censoring()}}, including \code{pt_data}, \code{obs_data0},
and \code{obs_data} components.}

\item{included_IDs}{A \link{character} \link{vector} of participant IDs to retain
in the filtered dataset.}
}
\value{
A list with the same structure as \code{dataset}, but with all
data frames filtered to include only rows where the ID is in
\code{included_IDs}.
}
\description{
Internal helper function that filters all data frames within a dataset
structure to include only the specified participant IDs.
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/plot_censoring_data.R
\name{plot_censoring_data}
\alias{plot_censoring_data}
\title{Plot censoring data}
\usage{
plot_censoring_data(
  dataset,
  included_IDs = unique(dataset$pt_data$ID),
  label_size = 5,
  point_size = 5,
  s_vjust = 2,
  labelled_IDs = included_IDs,
  xmin = min(dataset$pt_data$E) - 28,
  xmax = max(dataset$obs_data$O)
)
}
\arguments{
\item{dataset}{output from \code{\link[=simulate_interval_censoring]{simulate_interval_censoring()}}}

\item{included_IDs}{\link{character} \link{vector} of IDs from \code{dataset} to include}

\item{label_size}{\link{numeric}:
passed to \code{\link[ggrepel:geom_text_repel]{ggrepel::geom_text_repel()}}'s \code{size} argument}

\item{point_size}{\link{numeric}: passed to \link[ggplot2:geom_point]{ggplot2::geom_point}'s \code{size} argument}

\item{s_vjust}{passed to \link[ggrepel:geom_text_repel]{ggrepel::geom_text_repel}'s \code{vjust} argument}

\item{labelled_IDs}{\link{character} \link{vector}
indicating which IDs to label events for}

\item{xmin}{minimum displayed value for x-axis}

\item{xmax}{maximum displayed value for x-axis}
}
\value{
a ggplot
}
\description{
Plot censoring data
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/build_event_date_possibilities_table.R
\name{build_event_date_possibilities_table}
\alias{build_event_date_possibilities_table}
\title{Build table of event date possibilities}
\usage{
build_event_date_possibilities_table(
  participant_level_data,
  bin_width = 1,
  omega_hat = build_omega_table(participant_level_data, bin_width = bin_width)
)
}
\arguments{
\item{participant_level_data}{a \link{data.frame} with columns:
\itemize{
\item ID: participant identifier
\item Stratum: indicator for which population stratum the participant belongs to
\item L: left censoring interval endpoint
\item R: right censoring interval endpoint
}}

\item{bin_width}{number of days per bin}

\item{omega_hat}{a \link{data.frame} from \code{\link[=build_omega_table]{build_omega_table()}} representing the
seroconversion hazard model.}
}
\value{
a \link{data.frame} with columns:
\itemize{
\item ID: participant identifier
\item Stratum: indicator for which population stratum the participant belongs to
\item S: possible seroconversion dates
}
}
\description{
Build table of event date possibilities
}
\examples{
library(dplyr)
simulate_interval_censoring()$pt_data |>
  mutate(Stratum = 1) |>
  build_event_date_possibilities_table()
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/seroconversion_inverse_survival_function.R
\name{seroconversion_inverse_survival_function}
\alias{seroconversion_inverse_survival_function}
\title{Inverse survival function for time-to-event variable with linear hazard
function}
\usage{
seroconversion_inverse_survival_function(u, e, hazard_alpha, hazard_beta)
}
\arguments{
\item{u}{a vector of seroconversion survival probabilities}

\item{e}{a vector of time differences between study start and enrollment
(in years)}

\item{hazard_alpha}{the instantaneous hazard of seroconversion on the study
start date}

\item{hazard_beta}{the change in hazard per year after study start date}
}
\value{
numeric vector of time differences between study start and
seroconversion (in years)
}
\description{
This function determines the seroconversion date corresponding to a provided
probability of survival. See \doi{10.1111/biom.13472},
Supporting Information, Section A.4.
}
\references{
Morrison, Laeyendecker, and Brookmeyer (2021).
"Regression with interval-censored covariates: Application to
cross-sectional incidence estimation".
Biometrics, \doi{10.1111/biom.13472}.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/graph_simulated_hazards.R
\name{graph_simulated_hazards}
\alias{graph_simulated_hazards}
\title{Plot simulated seroconversion hazard curves}
\usage{
graph_simulated_hazards()
}
\value{
a \link[ggplot2:ggplot]{ggplot2::ggplot}
}
\description{
Plots example linear seroconversion hazards for several intercept/slope
combinations.
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/update_possible_subj_data.R
\name{update_possible_subj_data}
\alias{update_possible_subj_data}
\title{Update subject-level data with posterior probabilities}
\usage{
update_possible_subj_data(
  obs_data_possibilities,
  MAA_model,
  participant_level_data,
  omega_hat
)
}
\arguments{
\item{obs_data_possibilities}{A \link{data.frame} containing all possible
observation-level data combinations, including ID, S (possible
seroconversion dates), and Y (MAA classifications).}

\item{MAA_model}{A fitted \code{\link[stats:glm]{stats::glm()}} model object used to predict
P(Y=1|T=t), the probability of MAA-positive status given time since
seroconversion.}

\item{participant_level_data}{A \link{data.frame} containing participant-level
information, including ID, Stratum, and P(S>=l|E=e) (the probability
that seroconversion occurs on or after the last negative test date).}

\item{omega_hat}{A \link{data.frame} containing estimated parameters for the
seroconversion date distribution model, including S, Stratum,
P(S=s|S>=s,E=e), and P(S>s|S>=s,E=e).}
}
\value{
A \link{data.frame} containing updated subject-level data with the
following columns:
\itemize{
\item \code{ID}: Participant identifier
\item \code{Stratum}: Stratification variable
\item \code{S}: Possible seroconversion date
\item \verb{P(Y=y|T=t)}: Probability of observed MAA outcomes given time since
seroconversion
\item \verb{P(S=s|E=e)}: Marginal probability of seroconversion at time s given
enrollment date
\item \code{P(S=s|e,l,r,o,y)}: Posterior probability of seroconversion at time s
given all observed data (used to estimate omega and theta)
\item \code{P(S>=s|e,l,r,o,y)}: Posterior probability that seroconversion occurs
on or after time s (used to estimate omega)
}
}
\description{
Internal helper function that updates subject-level data by computing
various conditional probabilities used in the EM algorithm for
fitting the joint model. This function combines observation-level
predictions with participant-level data and omega estimates to compute
posterior probabilities needed for parameter estimation.
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/plot_CDF.R
\name{plot_CDF}
\alias{plot_CDF}
\title{plot estimated and true CDFs for seroconversion date distribution}
\usage{
plot_CDF(true_hazard_alpha, true_hazard_beta, omega_hat)
}
\arguments{
\item{true_hazard_alpha}{The data-generating hazard at the start of the study}

\item{true_hazard_beta}{The change in data-generating hazard per calendar
year}

\item{omega_hat}{tibble of estimated discrete hazards}
}
\value{
a ggplot
}
\description{
plot estimated and true CDFs for seroconversion date distribution
}
\examples{
\dontrun{

hazard_alpha <- 1
hazard_beta <- 0.5
study_data <- simulate_interval_censoring(
  "hazard_alpha" = hazard_alpha,
  "hazard_beta" = hazard_beta
)

# fit model:
EM_algorithm_outputs <- fit_joint_model(
  obs_level_data = study_data$obs_data,
  participant_level_data = study_data$pt_data
)
plot1 <- plot_CDF(
  true_hazard_alpha = hazard_alpha,
  true_hazard_beta = hazard_beta,
  omega_hat = EM_algorithm_outputs$Omega
)

print(plot1)
}

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/rwicc.R
\docType{package}
\name{rwicc}
\alias{rwicc}
\title{rwicc: Regression with Interval-Censored Covariates}
\description{
The \code{rwicc} package implements a regression model with an
interval-censored covariate using an EM algorithm, as described in
Morrison et al (2021); \doi{10.1111/biom.13472}.
}
\section{rwicc functions}{

The main \code{rwicc} functions are:
\itemize{
\item \code{\link{simulate_interval_censoring}}
\item \code{\link{fit_joint_model}}
}
}

\references{
Morrison, Laeyendecker, and Brookmeyer (2021).
"Regression with interval-censored covariates: Application to
cross-sectional incidence estimation".
Biometrics. \doi{10.1111/biom.13472}.
}
\seealso{
Useful links:
\itemize{
  \item \url{https://d-morrison.github.io/rwicc/}
  \item \url{https://github.com/d-morrison/rwicc}
  \item Report bugs at \url{https://github.com/d-morrison/rwicc/issues}
}

}
\author{
\strong{Maintainer}: Douglas Ezra Morrison \email{demorrison@ucdavis.edu} (\href{https://orcid.org/0000-0002-7195-830X}{ORCID}) [copyright holder]

Authors:
\itemize{
  \item Douglas Ezra Morrison \email{demorrison@ucdavis.edu} (\href{https://orcid.org/0000-0002-7195-830X}{ORCID}) [copyright holder]
  \item Ron Brookmeyer
}

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/seroconversion_survival_function.R
\name{seroconversion_density_function}
\alias{seroconversion_density_function}
\title{Seroconversion density function}
\usage{
seroconversion_density_function(...)
}
\arguments{
\item{...}{arguments passed to \code{\link[=seroconversion_survival_function]{seroconversion_survival_function()}} and
\code{\link[=seroconversion_hazard_function]{seroconversion_hazard_function()}}}
}
\value{
a numeric vector of density values
}
\description{
Probability density, \code{survival * hazard}.
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/utils.R
\name{build_phi_function_from_coefs}
\alias{build_phi_function_from_coefs}
\title{convert a pair of simple logistic regression coefficients into P(Y|T) curve:}
\usage{
build_phi_function_from_coefs(coefs)
}
\arguments{
\item{coefs}{numeric vector of coefficients}
}
\value{
function(t) P(Y=1|T=t)
}
\description{
convert a pair of simple logistic regression coefficients into P(Y|T) curve:
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/utils.R
\name{compute_mu}
\alias{compute_mu}
\title{compute mean window period duration from simple logistic regression
coefficients}
\usage{
compute_mu(theta)
}
\arguments{
\item{theta}{numeric vector of coefficients}
}
\value{
numeric scalar: mean window period duration
}
\description{
compute mean window period duration from simple logistic regression
coefficients
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/check_pt_data.R
\name{check_pt_data}
\alias{check_pt_data}
\title{Validate participant-level data}
\usage{
check_pt_data(participant_level_data)
}
\arguments{
\item{participant_level_data}{A data.frame or tibble containing
participant-level data with at least the following columns:
\itemize{
\item \code{ID}: participant ID
\item \code{L}: date of last negative test for seroconversion
\item \code{R}: date of first positive test for seroconversion
}}
}
\value{
NULL (invisibly). The function throws an error if validation fails.
}
\description{
Internal helper function that performs basic validation checks on
participant-level data to ensure data integrity before model fitting.
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/fit_joint_model.R
\name{fit_joint_model}
\alias{fit_joint_model}
\title{Fit a logistic regression model with an interval-censored covariate}
\usage{
fit_joint_model(
  participant_level_data,
  obs_level_data,
  model_formula = stats::formula(Y ~ T),
  mu_function = compute_mu,
  bin_width = 1,
  denom_offset = 0.1,
  EM_toler_loglik = 0.1,
  EM_toler_est = 1e-04,
  EM_max_iterations = Inf,
  glm_tolerance = 1e-07,
  glm_maxit = 20,
  initial_S_estimate_location = 0.25,
  coef_change_metric = "max abs rel diff coefs",
  verbose = FALSE
)
}
\arguments{
\item{participant_level_data}{a data.frame or tibble with the following
variables:
\itemize{
\item ID: participant ID
\item E: study enrollment date
\item L: date of last negative test for seroconversion
\item R: date of first positive test for seroconversion
\item Cohort` (optional): this variable can be used to stratify the modeling of
the seroconversion distribution.
}}

\item{obs_level_data}{a data.frame or tibble with the following variables:
\itemize{
\item ID: participant ID
\item O: biomarker sample collection dates
\item Y: MAA classifications (binary outcomes)
}}

\item{model_formula}{the functional form for the regression model for
p(y|t) (as a formula() object)}

\item{mu_function}{a function taking a vector of regression coefficient
estimates as input and outputting an estimate of mu (mean duration of
MAA-positive infection).}

\item{bin_width}{the number of days between possible seroconversion dates
(should be an integer)}

\item{denom_offset}{an offset value added to the denominator of the hazard
estimates to improve numerical stability}

\item{EM_toler_loglik}{the convergence cutoff for the log-likelihood
criterion ("Delta_L" in the paper)}

\item{EM_toler_est}{the convergence cutoff for the parameter estimate
criterion ("Delta_theta" in the paper)}

\item{EM_max_iterations}{the number of EM iterations to perform before
giving up if still not converged.}

\item{glm_tolerance}{the convergence cutoff for the glm fit in the M step}

\item{glm_maxit}{the iterations cutoff for the glm fit in the M step}

\item{initial_S_estimate_location}{determines how seroconversion date is
guessed to initialize the algorithm; can be any decimal between 0 and 1;
0.5 = midpoint imputation, 0.25 = 1st quartile, 0 = last negative, etc.}

\item{coef_change_metric}{a string indicating the type of parameter estimate
criterion to use:
\itemize{
\item "max abs rel diff coefs" is the "Delta_theta" criterion
described in the paper.
\item "max abs diff coefs" is the maximum absolute change in
the coefficients (not divided by the old values); this criterion can be
useful when some parameters are close to 0.
\item "diff mu" is the absolute change in mu,
which may be helpful in the incidence estimate calibration setting but not
elsewhere.
}}

\item{verbose}{whether to print algorithm progress details to the console}
}
\value{
a list with the following elements:
\itemize{
\item \code{Theta}: the estimated regression coefficients for the model of p(Y|T)
\item \code{Mu}: the estimated mean window period (a transformation of \code{Theta})
\item \code{Omega}: a table with the estimated parameters for the model of p(S|E).
\item \code{converged}: indicator of whether the algorithm reached its cutoff
criteria before reaching the specified maximum iterations. 1 = reached
cutoffs, 0 = not.
\item \code{iterations}: the number of EM iterations completed before the
algorithm stopped.
\item \code{convergence_metrics}: the four convergence metrics
\item \code{convergence_stats}: a table of the log-likelihood at each iteration
}
}
\description{
This function fits a logistic regression model for a binary outcome Y with an
interval-censored covariate T, using an EM algorithm, as described in
Morrison et al (2021); \doi{10.1111/biom.13472}.
}
\examples{
\dontrun{

# simulate data:
study_data <- simulate_interval_censoring()

# fit model:
EM_algorithm_outputs <- fit_joint_model(
  obs_level_data = study_data$obs_data,
  participant_level_data = study_data$pt_data
)
}
}
\references{
Morrison, Laeyendecker, and Brookmeyer (2021).
"Regression with interval-censored covariates: Application to
cross-sectional incidence estimation".
Biometrics. \doi{10.1111/biom.13472}.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/simulate_interval_censoring.R
\name{simulate_interval_censoring}
\alias{simulate_interval_censoring}
\title{Simulate a dataset with interval-censored seroconversion dates}
\usage{
simulate_interval_censoring(
  study_cohort_size = 4500,
  probability_of_ever_seroconverting = 0.05,
  n_at_risk = stats::rbinom(n = 1, size = study_cohort_size, prob =
    probability_of_ever_seroconverting),
  hazard_alpha = 1,
  hazard_beta = 0.5,
  preconversion_interval_length = 84,
  theta = c(0.986, -3.88),
  years_in_study = 10,
  max_scheduling_offset = 7,
  days_from_study_start_to_recruitment_end = 365,
  study_start_date = lubridate::ymd("2001-01-01")
)
}
\arguments{
\item{study_cohort_size}{the number of participants to simulate (N_0 in the
paper)}

\item{probability_of_ever_seroconverting}{the probability that each
participant is at risk of HIV seroconversion}

\item{n_at_risk}{number of participants who are at risk of infection; by
default, this number is determined stochastically from
\code{study_cohort_size} and \code{probability_of_ever_seroconverting}.}

\item{hazard_alpha}{the hazard (instantaneous risk) of seroconversion at the
start date of the cohort study for those participants at risk of
seroconversion}

\item{hazard_beta}{the change in hazard per calendar year}

\item{preconversion_interval_length}{the number of days between tests for
seroconversion}

\item{theta}{the parameters of a logistic model (with linear functional from)
specifying the probability of MAA-positive biomarkers as a function of time
since seroconversion}

\item{years_in_study}{the duration of follow-up for each participant}

\item{max_scheduling_offset}{the maximum divergence of pre-seroconversion
followup visits from the prescribed schedule}

\item{days_from_study_start_to_recruitment_end}{the length of the recruitment
period}

\item{study_start_date}{the date when the study starts recruitment ("d_0" in
the main text). The value of this parameter does not affect the simulation
results; it is only necessary as a reference point for generating E, L,
R, O, and S.}
}
\value{
A list containing the following two tibbles:
\itemize{
\item \code{pt_data}: a tibble of participant-level information, with the following
columns:
\itemize{
\item \code{ID}: participant ID
\item \code{E}: enrollment date
\item \code{L}: date of last HIV test prior to seroconversion
\item \code{R}: date of first HIV test after seroconversion
}
\item \code{obs_data}: a tibble of longitudinal observations with the following
columns:
\itemize{
\item \code{ID}: participant ID
\item \code{O}: dates of biomarker sample collection
\item \code{Y}: MAA classifications of biomarker samples
}
}
}
\description{
\code{simulate_interval_censoring} generates a simulated data set from a
data-generating model based on the typical structure of a cohort study of HIV
biomarker progression, as described in Morrison et al (2021);
\doi{10.1111/biom.13472}.
}
\examples{
study_data <- simulate_interval_censoring()
participant_characteristics <- study_data$pt_data
longitudinal_observations <- study_data$obs_data
}
\references{
Morrison, Laeyendecker, and Brookmeyer (2021).
"Regression with interval-censored covariates: Application to
cross-sectional incidence estimation".
Biometrics. \doi{10.1111/biom.13472}.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/build_omega_table.R
\name{build_omega_table}
\alias{build_omega_table}
\title{Build table of dates with possible seroconversions in dataset}
\usage{
build_omega_table(participant_level_data, bin_width = 1)
}
\arguments{
\item{participant_level_data}{a \link{data.frame} with columns:
\itemize{
\item Stratum: indicator for which population stratum the participant belongs to
\item L: left censoring interval endpoint
\item R: right censoring interval endpoint
}}

\item{bin_width}{number of days per bin}
}
\value{
a \link{data.frame}
}
\description{
Build table of dates with possible seroconversions in dataset
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/graph_simulated_survival_curves.R
\name{graph_simulated_survival_curves}
\alias{graph_simulated_survival_curves}
\title{Plot simulated seroconversion survival curves}
\usage{
graph_simulated_survival_curves()
}
\value{
a \link[ggplot2:ggplot]{ggplot2::ggplot}
}
\description{
Plots example seroconversion-time survival curves for several
intercept/slope hazard combinations.
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/seroconversion_survival_function.R
\name{seroconversion_cumhaz_function}
\alias{seroconversion_cumhaz_function}
\title{Seroconversion cumulative hazard function}
\usage{
seroconversion_cumhaz_function(t, intercept, slope, entry_time = 0)
}
\arguments{
\item{t}{numeric vector of times since study start (years)}

\item{intercept}{hazard at study start}

\item{slope}{change in hazard per year}

\item{entry_time}{time of study entry; the hazard is zero before this}
}
\value{
a numeric vector of cumulative hazard values
}
\description{
Cumulative hazard obtained by integrating \code{\link[=seroconversion_hazard_function]{seroconversion_hazard_function()}}.
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/seroconversion_survival_function.R
\name{seroconversion_survival_function}
\alias{seroconversion_survival_function}
\title{Seroconversion survival function}
\usage{
seroconversion_survival_function(...)
}
\arguments{
\item{...}{arguments passed to \code{\link[=seroconversion_cumhaz_function]{seroconversion_cumhaz_function()}}}
}
\value{
a numeric vector of survival probabilities
}
\description{
Survival probability, \verb{exp(-cumulative hazard)}.
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/fit_uniform_model.R
\name{fit_uniform_model}
\alias{fit_uniform_model}
\title{Fit model using uniform imputation}
\usage{
fit_uniform_model(
  participant_level_data,
  obs_level_data,
  maxit = 1000,
  tolerance = 1e-08,
  n_imputations = 10
)
}
\arguments{
\item{participant_level_data}{a data.frame or tibble with the following
variables:
\itemize{
\item ID: participant ID
\item E: study enrollment date
\item L: date of last negative test for seroconversion
\item R: date of first positive test for seroconversion
\item Cohort` (optional): this variable can be used to stratify the
modeling of the seroconversion distribution.
}}

\item{obs_level_data}{a data.frame or tibble with the following variables:
\itemize{
\item ID: participant ID
\item O: biomarker sample collection dates
\item Y: MAA classifications (binary outcomes)
}}

\item{maxit}{maximum iterations, passed to \code{bigglm}}

\item{tolerance}{convergence criterion, passed to \code{bigglm}}

\item{n_imputations}{number of imputed data sets to create}
}
\value{
a vector of logistic regression coefficient estimates
}
\description{
Fit model using uniform imputation
}
\examples{
sim_data <- simulate_interval_censoring(
  "theta" = c(0.986, -3.88),
  "study_cohort_size" = 4500,
  "preconversion_interval_length" = 365,
  "hazard_alpha" = 1,
  "hazard_beta" = 0.5
)

theta_est_midpoint <- fit_uniform_model(
  obs_level_data = sim_data$obs_data,
  participant_level_data = sim_data$pt_data
)

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/graph_S.R
\name{graph_S}
\alias{graph_S}
\title{Plot the estimated seroconversion-date distribution for one participant}
\usage{
graph_S(subject_level_data_possibilities, id = 1)
}
\arguments{
\item{subject_level_data_possibilities}{a \link{data.frame} of per-subject
seroconversion-date possibilities with their estimated probabilities}

\item{id}{the participant ID to plot}
}
\value{
a \link[ggplot2:ggplot]{ggplot2::ggplot}
}
\description{
Plot the estimated seroconversion-date distribution for one participant
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/standard_ggplot.R
\name{standard_ggplot}
\alias{standard_ggplot}
\title{Create a ggplot with standard theme settings}
\usage{
standard_ggplot(data)
}
\arguments{
\item{data}{A data frame to be passed to \code{\link[ggplot2:ggplot]{ggplot2::ggplot()}}.}
}
\value{
A \code{\link[ggplot2:ggplot]{ggplot2::ggplot()}} object with standard theme settings applied,
including a black and white theme, custom axis styling, and legend
positioning at the bottom.
}
\description{
Internal helper function that initializes a ggplot object with consistent
theme and styling settings used across multiple plotting functions in the
package.
}
\keyword{internal}
